#!/usr/bin/php -f
<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2006-02-27
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */


define ('GREEN', "\033[01;32m" );
define ('NORMAL', "\033[00m" );
define ('RED', "\033[01;31m" );

function install($OS) {
	if (!run("php check-deps.php", true)) {
		echo RED.'Not all the necessary dependencies were found. aborting.'.NORMAL."\n";
		exit(1);
	}

	$mailmandir = '/usr/lib/mailman';
	$mailmanvar = '/var/lib/mailman';
	$mailmancgi = $mailmandir.'/cgi-bin';
	$mailmanpub = $mailmanvar.'/archives/public';

	if ($OS == 'suse') {
		$apachegrp = 'www';
		$apacheusr = 'wwwrun';
	} else if ($OS == 'solaris') {
		$apachegrp = 'nobody';
		$apacheusr = 'nobody';
		$mailmandir = '/usr/local/mailman';
		$mailmanvar = '/usr/local/mailman';
		$mailmancgi = $mailmandir.'/cgi-bin';
		$mailmanpub = $mailmanvar.'/archives/public';

	} else if ($OS == 'ubuntu' || $OS=='deb') {
		$apachegrp = 'www-data';
		$apacheusr = 'www-data';
		$mailmandir = '/usr/lib/mailman';
		$mailmanvar = '/var/lib/mailman';
		$mailmancgi = '/usr/lib/cgi-bin/mailman';
		$mailmanpub = $mailmanvar.'/archives/public';
	} else {
		$apachegrp = 'apache';
		$apacheusr = 'apache';
	}
	
	// Where the PHP code will live
	$gforge_lib_dir = '/opt/gforge5';
	if (!is_dir($gforge_lib_dir)) {
		show("Error: GForge5 folder doesn't exist. Run install-gforge-1-deps.php first.");
		die();
	}
	
	// Where the data directories will live
	$gforge_var_dir = getenv('GFORGE_VAR_DIR');
	if (empty($gforge_var_dir)) {
		// RedHat
		$gforge_var_dir='/var/lib/gforge';
	}
	
	// Where the configuration files will live
	$gforge_etc_dir = getenv('GFORGE_ETC_DIR');
	if (empty($gforge_etc_dir)) {
		// RedHat
		$gforge_etc_dir = '/etc/gforge';
	}
	
	// Do not use SSL
	$use_ssl = 'false';
	// Do not use LDAP
	$use_ldap = 'false';
	
	show("\n-=# Welcome to GForge Post-Installer v5.0 #=-");

	show(" * Enter the Apache group ($apachegrp): ");
	$apache_group = trim(fgets(STDIN));
	if (strlen($apache_group) == 0) {
		$apache_group = $apachegrp;
		show(" ...using '$apache_group'");
	}
	
	show(" * Enter the Apache username ($apacheusr): ");
	$apache_user = trim(fgets(STDIN));
	if (strlen($apache_user) == 0) {
		$apache_user = $apacheusr;
		show(" ...using '$apache_user'");
	}
	
	show(' * Enter the Hostname for GForge (example: gforge5.company.com): ');
	$hostname = trim(fgets(STDIN));
	if (strlen($hostname) == 0) {
		$hostname = 'gforge5.company.com';
		show(" ...using '$hostname'");
	}
	
	show(' * Enter the Name for your GForge Install (example: MyGForge): ');
	$gforge_name = str_replace("'", '', trim(fgets(STDIN)));
	if (strlen($gforge_name) == 0) {
		$gforge_name = 'MyGForge';
		show(" ...using '$gforge_name'");
	}
	
	if (!is_dir($gforge_var_dir)) {
		mkdir($gforge_var_dir);
	}
	if (!is_dir($gforge_var_dir.'/filesystem')) {
		mkdir($gforge_var_dir.'/filesystem');
	}
	if (!is_dir($gforge_var_dir.'/caches')) {
		mkdir($gforge_var_dir.'/caches');
	}
	
	if (!is_dir($gforge_var_dir.'/exportProjects')) {
		mkdir($gforge_var_dir.'/exportProjects');
	}
	
	$sess_key = makeSessionKey(50);
	$gforge_conf = array("\$config['libDir'] *="=>			"\$config['libDir']='$gforge_lib_dir';", 
						 "\$config['etcDir'] *="=>			"\$config['etcDir']='$gforge_etc_dir';",
						 "\$config['varDir'] *="=>			"\$config['varDir']='$gforge_var_dir';",
						 "\$config['sessionKey'] *="=>		"\$config['sessionKey']='$sess_key';",
						 "\$config['apacheUser'] *="=>		"\$config['apacheUser']='$apache_user';",
						 "\$config['apacheGroup'] *="=>		"\$config['apacheGroup']='$apache_group';",
						 "\$config['hostName'] *="=>		"\$config['hostName']='$hostname';",
						 "\$config['name'] *="=>			"\$config['name']='$gforge_name';",
						 "\$config['use_ssl'] *="=>			"\$config['use_ssl']=$use_ssl;",
						 "\$config['usesLdap'] *="=>		"\$config['usesLdap']=$use_ldap;");
	
	exec('/bin/cp conf/gforge.conf.example '.$gforge_etc_dir.'/gforge.conf');
	makeConfig($gforge_etc_dir.'/gforge.conf', $gforge_conf);
	
	#svnroot
	if (!is_dir($gforge_var_dir.'/svnroot')) {
		mkdir($gforge_var_dir.'/svnroot');
	}
	exec("/bin/touch $gforge_var_dir/svnroot-authfile");
	if (!is_dir('/svnroot')) {
		exec('cd / && ln -s '.$gforge_var_dir.'/svnroot /svnroot');
	}
	
	#cvsroot
	if (!is_dir($gforge_var_dir.'/cvsroot')) {
		mkdir($gforge_var_dir.'/cvsroot');
	}
	if (!is_dir('/cvsroot')) {
		exec('cd / && ln -s '.$gforge_var_dir.'/cvsroot /cvsroot');
	}
	
	#permission setting
	run("chown -R $apache_user:$apache_group $gforge_lib_dir");
	run("chown -R $apache_user:$apache_group $gforge_etc_dir");
	run("chown -R $apache_user:$apache_group $gforge_var_dir/svnroot");
	//exec("chown -R $apache_user:$apache_group $gforge_var_dir/docsvnroot");
	run("chown -R $apache_user:$apache_group $gforge_var_dir/caches");
	run("chown -R $apache_user:$apache_group $gforge_var_dir/filesystem");
	
	#plugin configs
	$plugins=array();
	$plugins[]='mailman';
	$plugins[]='scmcvs';
	$plugins[]='scmsvn';
	$plugins[]='build';
	$plugins[]='wiki';
	$plugins[]='forum';
	$plugins[]='scmgit';
	
	if (!is_dir($gforge_etc_dir.'/plugins/')) {
		mkdir($gforge_etc_dir.'/plugins/');
	}
	for ($i=0; $i<count($plugins); $i++) {
		if (!is_dir($gforge_etc_dir.'/plugins/'.$plugins[$i])) {
			mkdir($gforge_etc_dir.'/plugins/'.$plugins[$i]);
		}
		exec('/bin/cp '.$gforge_lib_dir.'/plugins/'.$plugins[$i].'/conf/'.$plugins[$i] .'.conf '.$gforge_etc_dir.'/plugins/'.$plugins[$i].'/');
	
		// Config Mailman
		if ($plugins[$i] == 'mailman') {
			$mailman_conf = array("\$config['mailmanDir'] *="=>"\$config['mailmanDir']='$mailmandir';",
								"\$config['mailmanListsDir'] *="=>"\$config['mailmanListsDir']='$mailmanvar';",
								"\$config['listsHost'] *="=>"\$config['listsHost']='$hostname';");
		    makeConfig($gforge_etc_dir.'/plugins/mailman/mailman.conf', $mailman_conf);
		    if (file_exists($mailmandir.'/Mailman/mm_cfg.py.SuSEconfig')) {
			run("cp -f $mailmandir/Mailman/mm_cfg.py.SuSEconfig $mailmandir/Mailman/mm_cfg.py", true);
		    }
		}
		
		// Config CVS
		else if ($plugins[$i] == 'scmcvs') {
			if (strtolower(php_uname('s')) == 'sunos') {
				$auxcvs = trim(system("cvs --version | awk '{ print $5 }' | head -n2"));
				$parts = explode('.', $auxcvs);
				$cvsversion = $parts[0].'.'.$parts[1];
			} else {
				$cvsversion = system("cvs --version | awk '{ print $5 }' | head -n2 | tail -n1 | awk -F. '{ print $1\".\"$2 }'");
			}
			$cvs_conf = array("\$config['binaryVersion'] *="=>"\$config['binaryVersion']='$cvsversion';",
							  "\$config['useSsl'] *="=>"\$config['useSsl']='$use_ssl';");
			makeConfig($gforge_etc_dir.'/plugins/scmcvs/scmcvs.conf', $cvs_conf);
		}
		if ($plugins[$i] == 'build') {
			$hash = makeSessionKey(40);
			$build_conf = array("\$config['hash'] *="=>"\$config['hash']='$hash';");
			makeConfig($gforge_etc_dir.'/plugins/build/build.conf', $build_conf);
		}
		
	}
	
	run("cp $gforge_lib_dir/plugins/scmcvs/bin/cvssh.pl /bin");
	run('chmod 755 /bin/cvssh.pl');
	
	if (!file_exists("$gforge_lib_dir/www/tabber.js")) {
		show(' * Creating tabber link...');
		run("cd $gforge_lib_dir/www && ln -s ../lib/external/tabber.js tabber.js", true);
	}
	if (!file_exists("$gforge_lib_dir/www/fckeditor")) {
		show(' * Creating fckeditor...');
		run("cd $gforge_lib_dir/www && ln -s ../lib/external/fckeditor fckeditor", true);
	}
	if (!file_exists("$gforge_lib_dir/www/datetimepicker.js")) {
		show(' * Creating datetimepicker...');
		run("cd $gforge_lib_dir/www && ln -s ../lib/external/datetimepicker.js datetimepicker.js", true);
	}
	if (!file_exists("$gforge_lib_dir/www/prototype.js")) {
		show(' * * Creating prototype framework link...');
		run("cd $gforge_lib_dir/www && ln -s ../lib/external/prototype.js prototype.js", true);
	}
	if (!file_exists("$gforge_lib_dir/www/scriptaculous.js")) {
		show(' * * Creating scriptaculous link...');
		run("cd $gforge_lib_dir/www && ln -s ../lib/external/scriptaculous.js scriptaculous.js", true);
	}
	if (!file_exists("$gforge_lib_dir/www/ddaccordion.js")) {
		show(' * * Creating ddaccordion link...');
		run("cd $gforge_lib_dir/www && ln -s ../lib/external/ddaccordion.js ddaccordion.js", true);
	}
	if (!file_exists("$gforge_lib_dir/www/jquery.js")) {
		show(' * * Creating jquery link...');
		run("cd $gforge_lib_dir/www && ln -s ../lib/external/jquery.js jquery.js", true);
	}
	if (!file_exists("$gforge_lib_dir/www/mktree.js")) {
		show(' * * Creating mktree link...');
		run("cd $gforge_lib_dir/www && ln -s ../lib/external/mktree.js mktree.js", true);
	}
	if (!file_exists("$gforge_lib_dir/www/selectbox.js")) {
		show(' * * Creating selectbox link...');
		run("cd $gforge_lib_dir/www && ln -s ../lib/external/selectbox.js selectbox.js", true);
	}
	run("chmod 755 $gforge_lib_dir/lib/external/viewvc/bin/cgi/viewvc.cgi");
	
	show(' * Setting file permissions...');
	// Now make sure all the files in bin directories are executable
	
	exec("find $gforge_lib_dir -name 'bin' -type d", $dirs);
	foreach ($dirs as $dir) {
		if ($OS == 'solaris') {
		    run("find $dir  -type f -exec chmod +x {} \\;");
		} else {
		    run("find $dir -maxdepth 1 -type f -exec chmod +x {} \\;");
		}
	}

	// make sure README files weren't set +x (there are README files inside bin/ dirs)
	run("find $gforge_lib_dir -name \"README\" -type f -exec chmod -x {} \\;");
	
	// Make /etc/smrsh links if needed
	show(' * Making /etc/smrsh links...');
	if (is_dir('/etc/smrsh')) {
		if (!file_exists('/etc/smrsh/php')) {
			run("cd /etc/smrsh/ && ln -s /usr/bin/php php");
		}
		if (!file_exists('/etc/smrsh/mail_gateway.php')) {
			run("cd /etc/smrsh/ && ln -s $gforge_lib_dir/bin/mail_gateway.php mail_gateway.php");
		}
	}
	
	show(' * Installing CVS pserver...');
	if (!is_dir('/etc/xinetd.d')) {
		show('/etc/xinetd.d doesn\'t exist. Skipping...');
	} else if (file_exists('/etc/xinetd.d/cvspserver')) {
		show('/etc/xinetd.d/cvspserver already exists. Skipping...');
	} else {
		$cvspserver = "service cvspserver
	{
	        disable         = no
	        socket_type     = stream
	        wait            = no
	        user            = root
	        log_type        = FILE /var/log/cvspserver
	        protocol        = tcp
	        log_on_failure  += USERID
	        port            = 2401
	        server          = /usr/bin/php
	        server_args     = -d include_path=\".:$gforge_lib_dir/lib/external:$gforge_lib_dir/lib:$gforge_lib_dir\" $gforge_lib_dir/plugins/scmcvs/bin/cvs-pserver.php
	}";
		file_put_contents('/etc/xinetd.d/cvspserver', $cvspserver);
	}
	
	$apacheconffiles=array();
	if (is_file('/etc/httpd/conf/httpd.conf')) {
		$apacheconffiles[]='/etc/httpd/conf/httpd.conf';
	} elseif (is_file('/opt/csw/apache2/etc/httpd.conf')) {
		$apacheconffiles[]='/opt/csw/apache2/etc/httpd.conf';
	} elseif (is_file('/etc/apache2/httpd.conf')) {
		$apacheconffiles[]='/etc/apache2/httpd.conf';
	} else {
		$apacheconffiles[]='/etc/apache2/sites-enabled/000-default';
	}

	foreach ($apacheconffiles as $apacheconffile) {
		show(' * Setting GForge Include For Apache...');
		system("grep \"^Include $gforge_etc_dir/httpd.conf\" $apacheconffile > /dev/null", $ret);
		if ($ret == 1) {
			run("echo \"Include $gforge_etc_dir/httpd.conf\" >> $apacheconffile");
		}
	}
	
	if (is_dir('/etc/selinux')) {
		show(' * Disabling SELiunx...');
		run("find /etc/selinux -type f -name \"config\" -exec perl -pi -e 's/^SELINUX *=.*/SELINUX=disabled/gi' {} \;");
		run('setenforce 0', true);
	}
	
	show(' * Changing Startup Configuration...');
	
	if ($OS == 'suse') {
		changeInitConfigSuSE();
	} elseif ($OS == 'solaris') {
		configSolaris();
		changeInitConfigSolaris();
	} else {
		if ($OS != 'ubuntu') {
			changeInitConfig();
		}
	}
	
	$inifiles=array();
	if (is_file('/etc/php.ini')) {
		$inifiles[]='/etc/php.ini';
	} else if (is_file('/opt/csw/php5/lib/php.ini')) {
		$inifiles[]='/opt/csw/php5/lib/php.ini';
	} elseif (is_file('/etc/php5/apache2/php.ini')) {
		$inifiles[]='/etc/php5/apache2/php.ini';
		$inifiles[]='/etc/php5/cli/php.ini';
		if (is_file('/etc/php5/cgi/php.ini')) {
			$inifiles[]='/etc/php5/cgi/php.ini';
		}
	}
	
	foreach ($inifiles as $inifile) {
		show(' * Setting PHP Memory Limit to 64MB...');
		setIni('memory_limit', '64M', $inifile);
	
		show(' * Setting PHP Maximum Upload File Size to 20MB...');
		setIni('upload_max_filesize', '20M', $inifile);
		
		show(' * Disabling magic_quotes_gpc...');
		setIni('magic_quotes_gpc', 'Off', $inifile);

	
		show(' * Setting PHP Include Paths...');
		$include_paths = str_replace('=', '', setPHPPaths($gforge_lib_dir, $inifile));
	}
	
	show(' * Configuring httpd.conf...');
	$httpd_conf = array('ServerName'=>					"ServerName	$hostname",
						'DocumentRoot'=>				"DocumentRoot	$gforge_lib_dir/www",
						'ScriptAlias	/mailman'=>"ScriptAlias	/mailman	$mailmancgi",
						'Alias	/pipermail'=>"Alias	/pipermail	$mailmanpub",
						'<Directory "/opt/gforge5/www">'=>"<Directory \"$gforge_lib_dir/www\">",
						'<Directory "/usr/lib/mailman/cgi-bin">'=>"<Directory \"$mailmancgi\">",
						'<Directory "/var/lib/mailman/archives/public">'=>"<Directory \"$mailmanpub\">",
						'SVNParentPath'=>				"SVNParentPath $gforge_var_dir/svnroot",
						'AuthzSVNAccessFile'=>			"AuthzSVNAccessFile $gforge_var_dir/svnroot-access",
						'AuthUserFile'=>				"AuthUserFile $gforge_var_dir/svnroot-authfile",
						'php_value'=>					"php_value	$include_paths");

	if ($OS == 'solaris') {
	    $httpd_conf['LoadModule dav_module'] = '#LoadModule dav_module';
	    $httpd_conf['LoadModule dav_svn_module'] = '#LoadModule dav_svn_module';
	    $httpd_conf['LoadModule authz_svn_module'] = '#LoadModule authz_svn_module';
	} else {
	    if (is_file('/usr/lib64/apache2/mod_dav.so')) {
			$httpd_conf['LoadModule dav_module'] = 'LoadModule dav_module		/usr/lib64/apache2/mod_dav.so';
 	    } else if (is_file('/usr/lib/apache2/mod_dav.so')) {
			$httpd_conf['LoadModule dav_module'] = 'LoadModule dav_module		/usr/lib/apache2/mod_dav.so';
	    }
   	    if (is_file('/usr/lib64/apache2/mod_dav_svn.so')) {
			$httpd_conf['LoadModule dav_svn_module'] = 'LoadModule dav_svn_module	/usr/lib64/apache2/mod_dav_svn.so';
 	    } else if (is_file('/usr/lib/apache2/mod_dav_svn.so')) {
			$httpd_conf['LoadModule dav_svn_module'] = 'LoadModule dav_svn_module	/usr/lib/apache2/mod_dav_svn.so';
	    }
   	    if (is_file('/usr/lib64/apache2/mod_authz_svn.so')) {
			$httpd_conf['LoadModule authz_svn_module'] = 'LoadModule authz_svn_module	/usr/lib64/apache2/mod_authz_svn.so';
 	    } else if (is_file('/usr/lib/apache2/mod_authz_svn.so')) {
			$httpd_conf['LoadModule authz_svn_module'] = 'LoadModule authz_svn_module	/usr/lib/apache2/mod_authz_svn.so';
	    }
	}
	
	exec('/bin/cp conf/httpd.conf.example '.$gforge_etc_dir.'/httpd.conf');
	
	foreach($httpd_conf as $key => $val) {
		$key = str_replace("/", "\\/", $key);
		$key = str_replace('"', '\\"', $key);
		$val = str_replace("/", "\\/", $val);
		$val = str_replace('"', '\\"', $val);
		run("perl -pi -e \"s/$key.*/$val/gi\" $gforge_etc_dir/httpd.conf");
	}
	
	show(' * Restarting Apache...');
	if (is_file('/etc/init.d/httpd')) {
		run('/etc/init.d/httpd restart',true);
	} elseif (is_file('/etc/init.d/apache2')) {
		run('/etc/init.d/apache2 restart',true);
	} elseif (is_file('/etc/init.d/cswapache2')) {
		run('/etc/init.d/cswapache2 restart',true);
	}

	$db_config = include($gforge_etc_dir.'/gforge5-db-conf.php');
	$gforge_db = $db_config['propel']['datasources']['gf5']['connection']['database'];
	
	show(' * Updating GForge DB');
	run("cd $gforge_lib_dir/db/pgsql && php db-updater.php");
	run("cd $gforge_lib_dir/db/pgsql && php db-updater.php");
	run("cd $gforge_lib_dir/bin && php create_config_cache.php");
	run("cd $gforge_lib_dir/bin && php create_lang_cache.php");
	run('su - postgres -c "psql '.$gforge_db.' -c \\"UPDATE scm_server SET hostname=\''.$hostname.'\' WHERE scm_type=\'CVS\' OR scm_type=\'SVN\';\\""');

	echo "\n".RED.'Be sure to read README.PERFORMANCE !'.NORMAL."\n";

	finish();
}

function setIni($find, $value, $phppath) {
	$phpini = file($phppath);
	$new_value = "$find = $value\n";
	$found = false;
	for($i=0;$i<count($phpini);$i++) {
		$line = $phpini[$i];
		if (substr($line, 0, strlen($find)) == $find) {
			$parts = explode('=', $line);
			if (isset($parts[1])) {
				$phpini[$i] = $new_value;
				$found = true;
			}
		}
	}
	if ($found === false) {
		$find = '; End';
		reset($phpini);
		for($i=0;$i<count($phpini);$i++) {
			$line = $phpini[$i];
			if (substr($line, 0, strlen($find)) == $find) {
				$phpini[$i] = $new_value;
				$found = true;
			}
		}
		$phpini[] = '; End:';
	}
	file_put_contents($phppath, $phpini);
}


function setPHPPaths($gforge_lib_dir, $phppath) {
	$new_paths = '';
	$include_paths = array('.', $gforge_lib_dir.'/lib/external', $gforge_lib_dir.'/lib', $gforge_lib_dir);
	$phpini = file($phppath);
	$find = 'include_path';
	$found = false;
	for($i=0;$i<count($phpini);$i++) {
		$line = str_replace('"', '', $phpini[$i]);
		if (substr($line, 0, strlen($find)) == $find) {
			$parts = explode('=', $line);
			if (isset($parts[1])) {
				$paths = explode(':', $parts[1]);
				foreach ($paths as $path) {
					$path = trim($path);
					if (array_search($path, $include_paths) === false) {
						$include_paths[] = $path;
					}
				}
				sort($include_paths);
				$new_paths = 'include_path = "'.implode(':', $include_paths).'"'."\n";
				$phpini[$i] = $new_paths;
				$found = true;
			}
		}
	}
	if ($found === false) {
		$find = '; End';
		reset($phpini);
		for($i=0;$i<count($phpini);$i++) {
			$line = str_replace('"', '', $phpini[$i]);
			if (substr($line, 0, strlen($find)) == $find) {
				$new_paths = 'include_path = "'.implode(':', $include_paths).'"'."\n";
				$phpini[$i] = $new_paths;
				$found = true;
			}
		}
		$phpini[] = '; End:';
	}
	file_put_contents($phppath, $phpini);
	return $new_paths;
}

function makeConfig($file, $config) {
	foreach($config as $key => $val) {
		$key = str_replace("/", "\\/", $key);
		$key = str_replace("'", "\\'", $key);
		$key = str_replace("[", "\\[", $key);
		$key = str_replace("]", "\\]", $key);
		$key = str_replace("$", "\\$", $key);
		
		$val = str_replace("/", "\\/", $val);
		$val = str_replace("'", "\\'", $val);
		$val = str_replace("[", "\\[", $val);
		$val = str_replace("$", "\\$", $val);
		$val = str_replace("]", "\\]", $val);
		run("perl -pi -e \"s/$key.*/$val/gi\" $file");
	}
}

function makeSessionKey($size) {
	$key = '';
	$values = '1234567890abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
	$max_num = (strlen($values) - 1);

	for($i = 0;$i < $size;$i++) {
		$pos = rand(0, $max_num);
		$key .= $values{$pos};
	}
	return $key;
}

function changeInitConfig() {
	run('service iptables stop', true);
	run('chkconfig iptables off', true);
	run('chkconfig httpd on', true);
	run('chkconfig postgresql on', true);
}

function changeInitConfigSuSE() {
	run('service SuSEfirewall2_init stop', true);
	run('service SuSEfirewall2_setup stop', true);
	run('chkconfig SuSEfirewall2_init off', true);
	run('chkconfig SuSEfirewall2_setup off', true);
	run('chkconfig apache2 on', true);
	run('chkconfig postgresql on', true);
}

function changeInitConfigSolaris() {
	run('/etc/init.d/cswapache2 restart', true);
	run('/etc/init.d/cswpostgres restart', true);
}

function configSolaris() {
	if (!file_exists('/bin/php'))
		run('ln -s /opt/csw/bin/php5 /bin/php', true);
	if (!file_exists('/usr/bin/python'))
		run('ln -s /opt/csw/bin/python /usr/bin/python', true);
	if (!file_exists('/usr/bin/svnadmin'))
		run('ln -s /opt/csw/bin/svnadmin /usr/bin/svnadmin', true);
	if (!file_exists('/usr/bin/cvs'))
		run('ln -s /opt/csw/bin/cvs /usr/bin/cvs', true);
	if (!file_exists('/usr/bin/svnlook'))
		run('ln -s /opt/csw/bin/svnlook /usr/bin/svnlook', true);
	if (!file_exists('/usr/bin/rcsdiff'))
		run('ln -s /opt/csw/bin/rcsdiff /usr/bin/rcsdiff', true);
	if (!file_exists('/usr/bin/rlog'))
		run('ln -s /opt/csw/bin/rlog /usr/bin/rlog', true);
	run('/usr/bin/crle -c /var/ld/ld.config -l /lib:/usr/lib:/opt/csw/lib:/opt/csw/bdb4/lib', true);
}

function finish() {
	show(NORMAL."Done.\nGForge5 has been successfully installed and configured.");
}

function show($text, $newLine = true) {
	if ($newLine) {
		$text = GREEN.$text .NORMAL."\n";
	}
	fwrite(STDOUT, $text);
}

function run($command, $ignore = false) {
	system($command, $ret);
	if ($ignore) {
		if ($ret != 0) {
			return false;
		} else {
			return true;
		}
	} else {
		if ($ret != 0) {
			echo RED.'An error ocurred running the last command... aborting.'.NORMAL."\n";
			die();
		}
	}
}

if (count($argv) < 2) {
    show('Usage: install-gforge-3.php [FC4|FC5|RHEL4|RHEL5|DEBIANETCH|UBUNTU6|UBUNTU7|UBUNTU9|CENTOS43|CENTOS5]');
} else {
    $platform = strtolower($argv[1]);

	if ($platform == 'fc3' || $platform == 'centos4') {
		install('fc3');
	} elseif ($platform == 'fc4' || $platform == 'fc5') {
		install('fc5');
	} elseif ($platform == 'rhel4' || $platform == 'centos43') {
		install('rhel4');
	} elseif ($platform == 'rhel5' || $platform == 'centos5') {
		install('rhel4');
	} elseif ($platform == 'debianetch') {
		install('deb');
	} elseif ($platform == 'suse10' || $platform == 'sles10') {
		install('suse');
	} elseif ($platform == 'ubuntu6' || $platform == 'ubuntu7' || $platform == 'ubuntu9') {
		install('ubuntu');
	} elseif ($platform == 'solaris10') {
		install('solaris');
	} else {
		echo RED."UNSUPPORTED PLATFORM".NORMAL."\n";
	}
}
?>
