<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
 
/**
 * A collection of system utilities
 */
class SystemUtils {
	function isAssociativeArray($array){
		if (!is_array($array)) return false;
			foreach(array_keys($array) as $key=>$value) {
				if ($key != $value) return true;
		}
		return false;
	}

	/**
	 * Given a directory and a file name, builds the full path to the
	 * filename. This avoids having file names like /etc/gforge//local.inc.
	 *
	 * @param	string $path	Directory name
	 * @param	string $filename	File name to be attached to $path (note that it can also be a directory name)
	 */
	static function buildFilePath($path, $filename) {
		$slashPath = preg_match("/\\/\$/", $path);
		$slashFile = preg_match("/^\\//", $filename);
		if ($slashPath && $slashFile) {
			$filename = preg_replace("/^\\/*/", "", $filename);
			return $path.$filename;
		} else if ($slashPath && !$slashFile) {
			return $path.$filename;
		} else if (!$slashPath && $slashFile) {
			return $path.$filename;
		} else if (!$slashPath && !$slashFile) {
			return $path."/".$filename;
		}
	}
	/**
	 * sendMail() - Sends a MIME email
	 *
	 * @param	string	The subject
	 * @param	string	The body
	 * @param mixed	To field
	 * @param boolean	Whether to send HTML email
	 * @param string	From field
	 * @param string	Reply-To field
	 * @param array		List of attached file location
	 * @return boolean	True on success, false on failure
	 */
	static function sendMail($subject,$body,$to,$html=false,$from='',$reply='', $attachment = array()) {

		if (empty($from)) {
			// default "from" address
			$from = self::getFromAddress();
		}
		if (empty($reply)) {
			$reply = $from;
		}

		if (is_array($to)) {
			$to_all = $to;
		} elseif ($to instanceof User) {
			$to_all = array($to->getEmail());
		} else {
			// it's a single e-mail address
			$to_all = array($to);
		}

		// Sanitize subject
		$subject = str_replace("\n", '', $subject);
		$charset = 'UTF-8';

	 	$sendmail = self::getSendmailPath();
		//$sendmail = '/usr/sbin/sendmail';

		if (!$sendmail) {
			return false;
		}

		$j = 0;

		foreach ($to_all as $to) {

			$uid = md5(uniqid(time()));
			$contents = '';
			//$contents .= "From: ".$from . "\r\n";		// This is included in the command with the -f parameter
			$contents .= "To: ".$to . "\r\n";
			$contents .= "Reply-To: ".$reply ."\r\n";
			$contents .= "Subject: " . "=?utf-8?B?".base64_encode($subject)."?=" . "\r\n";
			
			if (count($attachment) == 0) {
				//TODO: Handle HTML
				$contents .= "Content-Type: text/plain; charset=UTF-8"."\r\n";
				$contents .= "\r\n";
				$contents .= $body;
			}  else {
				// Email with attachments
				$contents .= "MIME-Version: 1.0\r\n";
				$contents .= "Content-Type: multipart/mixed; boundary=\"".$uid."\"\r\n";
				$contents .= "\r\n";
				$contents .= "This is a multi-part message in MIME format.\r\n";
				$contents .= "--".$uid."\r\n";
				$contents .= "Content-Type: text/plain; charset=UTF-8"."\r\n";
				$contents .= "Content-Transfer-Encoding: 7bit\r\n";
				$contents .= "\r\n";
				$contents .= $body;
				$contents .= "\r\n\r\n";

				// Include the attachments
				for ($i=0; $i < count($attachment); $i++){
					$file = $attachment[$i];
					$name = basename($file);
					$file_size = filesize($file);
					//echo $file . " " . $file_size . "\n";
					$handle = fopen($file, "r");
					if (!$handle) {
						continue;
					}
					$file_content = fread($handle, $file_size);
					fclose($handle);
					$file_content = chunk_split(base64_encode($file_content));

					$contents .= "--".$uid."\r\n";
					$contents .= "Content-Type: application/octet-stream; name=\"".$name."\"\r\n";
					$contents .= "Content-Transfer-Encoding: base64\r\n";
					$contents .= "Content-Disposition: attachment; filename=\"".$name."\"\r\n\r\n";
					$contents .= $file_content."\r\n\r\n";
				}
				$contents .= "--".$uid."--";
			}

			$prep_body = self::prepStringForSendmail($contents);

			// Execute sendmail
			$cmd = '/bin/echo '.$prep_body.' | '.$sendmail.' -f\''.$from.'\' -t -i > /dev/null 2>&1 &';

			$output = array();
			$error = self::execute($cmd, $output);

			if ($error) {
				return false;
			}

			$j++;
			if (($j % 15) == 0) {
				// Pause every 15 emails. This will prevent the server from hanging.
				sleep(5);
			}
		}
		
		return true;
	}
	
	/**
	 * Returns an email address with the name of the GForge server and the host name
	 */
	public static function getFromAddress($name='noreply') {
		if (GFConf::get('system.simpleFromEmail')) {
			return $name.'@'.GFConf::get('system.hostName');
		} else {
			return '"GForge" <'.$name.'@'.GFConf::get('system.hostName').'>';
		}
	}
	
	private static function prepStringForSendmail($str) {
		/*$str = str_replace("`","\\`",$str);
		$str = str_replace("\"","\\\"",$str);
		$str = str_replace("\$","\\\$",$str);
		*/
		$str = escapeshellarg($str);
		return $str;
	}
	
	public static function convertToCharset($string, $charset) {
		if (!function_exists('mb_convert_encoding')) {
			return $string;
		}
		
		$res = '=?'.$charset.'?B?';
		$res .= base64_encode(mb_convert_encoding($string, $charset, 'UTF-8'));
		$res .= '?=';
		
		return $res;
	}
	
	/**
	 * Tries to guess the system's sendmail path. Returns false if sendmail not found.
	 * @return	string
	 */
	private static function getSendmailPath() {
		static $sendmail = null;
		if (is_null($sendmail)) {

			if (GFConf::varExists('system.sendmailPath')) {
				$sendmail = GFConf::get('system.sendmailPath');
			} else {
				$sendmail = trim(`which sendmail`);
				if (!$sendmail) {
					// Default value
					$sendmail = '/usr/sbin/sendmail';
				}
			}

			if (!file_exists($sendmail)) {
				$sendmail = false;
			}
		}
		
		return $sendmail;
	}
	
	public static function sendUserMail($subject, $body, $html=false, $from='', $reply='') {
		$session = Session::getSession();
		if (!$session->isLoggedIn()) {
			throw new Exception('Cannot send email: not logged in');
		}
		$user = $session->getUser();
		$email = $user->getEmail();
		self::sendMail($subject, $body, $email, $html, $from, $reply);
	}
	
	/**
	* isRootDir() - Checks if a directory points to the root dir
	* @param	string	Directory
	* @return bool
	*/
	public static function isRootDir($dir) {
		return !preg_match('/[^\\/]/',$dir);
	}
	
	/**
	 * Execute a command
	 * @param	string $command	Command to execute
	 * @param	array &$output	Array to store the lines of the output
	 * @param	bool	Whether the output should be stored in an array, line by line. Could cause problems with binary files.
	 * @return	int	Result of the command
	 */
	public static function execute($command, &$output, $explode_output=true) {
		$retval = 0;
		if ($explode_output) {
			exec($command, $output, $retval);
		} else {
			$output = "";
			$ph = popen($command, 'r');
			while ($buffer = fread($ph, 4096)) {
				$output .= $buffer;
			}
			pclose($ph);
			$retval = 0;		// No way to catch errors with popen :(
		}
		return $retval;		
	}
	
	/**
	 * Checks if a given group exists in the system
	 * @param	string
	 * @return	bool
	 */
	public static function groupExists($group_name) {
		$group_name = escapeshellarg($group_name);
		return (exec('getent group '.$group_name) != '');
	}
	
	/**
	 * Checks if a given user exists in the system
	 * @param	string
	 * @return	bool
	 */
	public static function userExists($user_name) {
		$user_name = escapeshellarg($user_name);
		return (exec('getent passwd '.$user_name) != '');
	}
	
	/**
	 * Returns the users's home dir
	 * @param	mixed	User
	 * @return	string
	 */
	public static function getHomedir($user) {
		if ($user instanceof User) {
			$user = $user->getUnixName();
		}
		
		return self::buildFilePath(GFConf::get('system.homeDir'), '/'.$user);
	}
	
	/**
	 * Initialize a user's home directory
	 * @param	mixed	User
	 */
	public static function initializeHomedir($user) {
		if ($user instanceof User) {
			$user = $user->getUnixName();
		}
		
		$homedir = self::getHomedir($user);
		if (is_dir($homedir)) return;
		
		if (!mkdir($homedir)) {
			throw new PermissionDeniedException('Couldn\'t create user\'s home directory: '.$homedir);
		}

		chown ($homedir, $user);
		chgrp ($homedir, 'users');
	}

	/**
	 * Format file size in a human-readable way
	 *
	 * @param int the size in bytes
	 * @return string human readable size
	 */
	public static function getReadableSize ($bytes,$round=0) { 		
		if($bytes < 1024) $hrb = $bytes.' bytes';
		if($bytes >=1024 && $bytes < 1048576) $hrb = round($bytes/1024,$round).' KB';
		if($bytes >= 1048576 && $bytes < 1073741824) $hrb = round($bytes/1048576,$round).' MB';
		if($bytes >= 1073741824) $hrb = round($bytes/1073741824,$round).' GB';
		return $hrb;
	}
	
	public static function getDirSize($path) {
		return trim(shell_exec("du -sb {$path};"), "\x00..\x2F\x3A..\xFF");
	}

	public static function makeUrlsClickable($string) {
		return (preg_replace( '/(ftp|http|https):\/\/(\w+:{0,1}\w*@)?(\S+)(:[0-9]+)?(\/|\/([\w#!:.?+=&%@!\-\/]))?/', '<a href="\0" target="_blank">\0</a>', $string));
	}
}
?>
