<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Marcelo Mottalli <marcelo@gforgegroup.com>
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('core/TroveLink.php');
require_once('core/TroveCategory.php');
require_once('Validator.php');

/**
 * This class manages the Trove forms
 */
abstract class TroveFilterRenderer {
	protected $section = null;
	
	public function __construct($section) {
		$this->section = $section;
	}
	
	public function getFilterForm() {
		$lang = Language::getSessLang();
		$url = GFUrl::getUrl();
		$theme = Session::getSession()->getTheme();
		
		$result = '';
		
		// Show filters, if any
		$trove_filters = $this->getTroveFilters();
		if (!empty($trove_filters)) {
			$result .= $lang->getText('Trove.FilterLimit').'<br />';
			$root_categories_names = TroveCategoryPeer::getRootCategoriesNames();
			
			$c = new Criteria();
			$c->add(TroveCategoryPeer::TROVE_CATEGORY_ID, $trove_filters, Criteria::IN);
			$filter_categories = TroveCategoryPeer::doSelect($c);
			$result.='<ul style="padding-left:10px;">';
			foreach ($filter_categories as $category) {
				$result .= '<li>';
				if (array_key_exists($category->getRootTroveCategoryId(), $root_categories_names)) {
					$result .= $root_categories_names[$category->getRootTroveCategoryId()].' - ';
				}
				$result .= $category->getCategoryName().' ';
				
				// Create the new filter
				$new_filter = array_diff($trove_filters, array($category->getTroveCategoryId()));		// Nice way to extract an element from an array
				$result .= '<a href="'.GFUrl::getUrl()->currentUrl(array('_trove_filter' => implode(',', $new_filter))).'">'.$lang->getText('Trove.RemoveFilter').'</a>';
				$result .= '</li>';
			}
			$result .= '</ul>';
			$result .= '<hr />';
		}
		
		// Get the root categories
		$root_categories = TroveCategoryPeer::getRootCategoriesNamesWithCount($this->section);

		// Get the information about the current selected branch (roots + children)
		if (!isset($_GET['_trove_category_id'])) {
			$trove_category_id = GFConf::get('system.troveTopic');
		} else {
			$trove_category_id = Validator::sanitizeInt($_GET['_trove_category_id']);
		}
		
		$c = new Criteria();
		$this->prepareCountJoinCriteria($c);
		$node = TroveCategoryPeer::getCategoryNodeWithCount($trove_category_id, $this->section, $c);
		if (!$node) {
			throw new Exception('Invalid category ID selected: '.$trove_category_id);
		}
		
		$branch = $node['branch'];
		
		// Level 0 of the branch is the root category
		$selected_root_category_id = $branch[0]['trove_category_id'];

		$result.='<ul style="padding-left:10px;">';
		foreach ($root_categories as $root_id => $root_data) {
			if ($root_id == $selected_root_category_id) {
				$result .= $this->displayCurrentBranch($node);
			} else {
				$root_name = $root_data['name'];
				$total_count = $root_data['category_count'] + $root_data['children_count'];
				// don't display empty categories
				if ($total_count == 0) {
					continue;
				} else {
					$result .= '<li><img src="'.$theme->getImgUrl('ic/ofolder16.png').'"/>&nbsp;<a href="'.$url->currentUrl(array('_trove_category_id' => $root_id)).'">'.$root_name.'</a></li>';
				}
			}
		}
		$result .= '</ul>';

		
		// Now search the elements in this category
		$c = new Criteria();
		$c->addJoin(TroveLinkPeer::TROVE_CATEGORY_ID, TroveCategoryPeer::TROVE_CATEGORY_ID);
		$c->add(TroveCategoryPeer::TROVE_CATEGORY_ID, $trove_category_id);
		$c->add(TroveLinkPeer::SECTION, $this->section);

		$result .= $this->getCategoryResults($c);
		
		return $result;
	}
	
	public function getItemTrove(&$trove, $show_filter=false) {
		$lang = Language::getSessLang();
		$filters = $this->getTroveFilters();
	
		$result = '';
		if (count($trove) > 0) {
			$result .= '<ul>';
			foreach ($trove as $trove_root => $data) {
				if (count($data['children']) > 0) {
					$result .= '<li>'.$data['name'].': ';
					$children_names = array();
					foreach ($data['children'] as $children) {
						$result .= $children->getCategoryName();
						if ($show_filter) {
							$category_id = $children->getTroveCategoryId();
							if (in_array($category_id, $filters)) {
								$result .= ' <strong>'.$lang->getText('Trove.Filtering').'</strong>';
							} else {
								$result .= ' <a href="'.$this->getAddFilterUrl($children->getTroveCategoryId()).'">'.$lang->getText('Trove.Filter').'</a>';
							}
						}
						$result .= ', ';
					}
					// Remove trailing comma
					$result = preg_replace('/, $/', '', $result);
					
					$result .= '</li>'."\n";
				}
			}
			$result .= '</ul>';
		} else {
			$result .= $lang->getText('Trove.NotTroveCat');
		}
		
		return $result;

	}
	
	protected function getAddFilterUrl($trove_category_id) {
		$url = GFUrl::getUrl();
		$filters = $this->getTroveFilters();
		if (!in_array($trove_category_id, $filters)) {
			$filters[] = $trove_category_id;
		}
		return $url->currentUrl(array('_trove_filter' => implode(',', $filters)));
	}
	
	protected function getTroveFilters() {
		if (isset($_GET['_trove_filter']) && strlen($_GET['_trove_filter'] > 0)) {
		} else {
			return array();
		}
		return (isset($_GET['_trove_filter'])) ? explode(',', $_GET['_trove_filter']) : array();
	}
	
	/**
	 * See if all the categories in the trove
	 * @param	array $trove		Array of trove categories for an item
	 */
	protected function matchesFilters(&$trove) {
		$filters = $this->getTroveFilters();
		$item_troves = array();
		foreach ($trove as $trove_root => $data) {
			foreach ($data['children'] as $children) {
				$item_troves[] = $children->getTroveCategoryId();
			}
		}
		
		// Now we must check that all the elements in $filters are in $item_troves
		$res = array_diff($filters, $item_troves);
		return empty($res);
	}
	
	protected function displayCurrentBranch(&$node) {
		$lang = Session::getSession()->getLanguage();
		$theme = Session::getSession()->getTheme();
		
		$result = '';

		$branch = $node['branch'];
		$children = $node['children'];

		// Display the category tree
		for ($level=0; $level < count($branch); $level++) {
			$category = $branch[$level];
			$is_root = ($level == 0);

			if (!$is_root) $result.='<ul style="padding-left:10px;">';

			if ($level == count($branch)-1) {
				// displaying current category (latest level)
				$result .= '<li style="font-weight:bold;"><img src="'.$theme->getImgUrl('ic/ofolder16.png').'"/>&nbsp;'.$category['category_name'].'';

				// Display the children
				$result .= '<ul style="padding-left:10px;">';
				foreach ($node['children'] as $children) {
					$child_count = $children['category_count'] + $children['children_count'];
					// If this category and all its subcategories are empty, don't display it
					if ($child_count == 0) {
						continue;
					}
					
					$result .= '<li style="font-weight:normal;"><img src="'.$theme->getImgUrl('ic/ofolder16.png').'"/>&nbsp;'.$children['category_name'].' ';
					$result .= '<a href="'.GFUrl::getUrl()->currentUrl(array('_trove_category_id' => $children['trove_category_id'])).'">'.$lang->getText('Trove.ShowCategory').'</a> ';
					if (in_array($children['trove_category_id'], $this->getTroveFilters())) {
						// Already filtering?
						$result .= '<strong>'.$lang->getText('Trove.Filtering').'</strong> ';
					} else {
						$result .= '<a href="'.$this->getAddFilterUrl($children['trove_category_id']).'">'.$lang->getText('Trove.Filter').'</a> ';
					}
					$result .= '<i>'.$this->getCountString($children['category_count'], $children['children_count']).'</i>';
					$result .= '</li>';
				}
				$result .= '</ul></li>';
			} else {
				// Displaying parent category
				$result .= '<li><img src="'.$theme->getImgUrl('ic/ofolder16.png').'"/>&nbsp;<a href="'.GFUrl::getUrl()->currentUrl(array('_trove_category_id' => $category['trove_category_id'])).'">'.$category['category_name'].'</a></li>';
			}

		}

		// Close the item lists. There are as many open lists as levels in the current branch
		for ($i=0; $i < count($branch)-1; $i++) {
			$result .= '</ul>';
		}
		
		return $result;
		
	}
	
	protected abstract function prepareCountJoinCriteria(Criteria $c);
	protected abstract function getCategoryResults(Criteria $c);
	protected abstract function getCountString($category_count, $children_count);

}
?>
