<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: ForumAdminFormRenderer.php 4415 2010-03-10 16:15:09Z tperdue $
 * @author	Marcelo Mottalli <marcelo@gforgegroup.com>
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once('SystemUtils.php');
require_once('GFConf.php');

class ForumAdminFormRenderer {
	private $section = null;
	private $ref_id = null;
	private $wwwlibPath;
	private $template;
	private $language;
	private $errors = array();

	public function __construct($section, $ref_id, $template, $language) {
		$this->section = $section;
		$this->ref_id = $ref_id;
		$this->template = $template;
		$this->language = $language;
		
		$this->wwwlibPath = SystemUtils::buildFilePath(GFConf::get('system.pluginsPath'), '/forum/wwwlib/admin/forms');
	}
	
	/**
	 * @return string
	 * @param	string	$redirect_to	URL to redirect on succesful add
	 * @param	bool	$force_form		Whether to force the display of the form (disables processing the POST data)
	 */
	public function addNewForumForm($redirect_to, $force_form=false) {
		// See if we've submitted the form. In that case, process the data.
		if (isset($_POST['forum_add_submit']) && !$force_form) {
			return $this->postAddNew($redirect_to);
		}
		
		$formFile = SystemUtils::buildFilePath($this->wwwlibPath, '/ForumAdd-tpl.php');
		$section=((isset($_POST['section'])) ? htmlspecialchars($_POST['section']) : '' );
		$ref_id=((isset($_POST['ref_id'])) ? htmlspecialchars($_POST['ref_id']) : '' );
		$forum_name=((isset($_POST['forum_name'])) ? htmlspecialchars($_POST['forum_name']) : '' );
		$description=((isset($_POST['description'])) ? htmlspecialchars($_POST['description']) : '' );
		$send_all_posts_to=((isset($_POST['send_all_posts_to'])) ? htmlspecialchars($_POST['send_all_posts_to']) : '' );
		$thread_count=((isset($_POST['thread_count'])) ? htmlspecialchars($_POST['thread_count']) : '' );
		$message_count=((isset($_POST['message_count'])) ? htmlspecialchars($_POST['message_count']) : '' );
		$most_recent_date=((isset($_POST['most_recent_date'])) ? htmlspecialchars($_POST['most_recent_date']) : '' );
		$moderation_level=((isset($_POST['moderation_level'])) ? htmlspecialchars($_POST['moderation_level']) : '' );
		if (isset($_POST['forum_edit_submit'])) {
			$show_associations_tab = (isset($_POST['show_associations_tab']) ? true : false);
			$show_tags_tab = (isset($_POST['show_tags_tab']) ? true : false);
		} else {
			$show_associations_tab = true;
			$show_tags_tab = true;
		}
		
		$section = $this->section;
		$ref_id = $this->ref_id;
		$theme = $this->template;
		$Lang = $this->language;
		
		ob_start();	
		include($formFile);
		$form = ob_get_clean();
		
		return $form;
	}
	
	/**
	 * @return string
	 */
	public function showForumBrowseForm() {
		$Lang = $this->language;
		$theme = $this->template;

		// Retrieve the objects array
		$crit = new Criteria();
		$crit->add(ForumPeer::SECTION, $this->section);
		$crit->add(ForumPeer::REF_ID, $this->ref_id);
		$crit->addAscendingOrderByColumn(ForumPeer::FORUM_ID);
		$arr = ForumPeer::doSelect($crit);
		
		$ret = '';
		
		$theader = array();
	
		$theader[] = $Lang->getText('Forum.ForumIdName');
		$theader[] = $Lang->getText('Forum.ForumNameName');
		$theader[] = $Lang->getText('Forum.DescriptionName');
		$theader[] = $Lang->getText('Forum.EditPageTitle');
		$theader[] = $Lang->getText('Forum.ApproveMessages');
		$theader[] = $Lang->getText('Forum.DeletePageTitle');
	
		$ret .= $theme->tableTop($theader);

		if (count($arr) < 1) {
			$ret .= $theme->tableFullRow('<span class="nonefound">'.$Lang->getText('Forum.NoneFound').'</span>');
		} else {
			foreach ($arr as $item) {
				$trow = array();
				$trow[]=$item->getForumId();
				$trow[]=$item->getForumName();
				$trow[]=$item->getDescription();				
				$trow[]= '<a href="'.GFUrl::getUrl()->currentBaseUrl('ForumEdit', array('id' => $item->getForumId())).'">'.$Lang->getText('Forum.EditPageName').'</a>';
				if ($item->getModerationLevel() > Forum::NO_MODERATION) {
					$trow[]= '<a href="'.GFUrl::getUrl()->currentBaseUrl('PendingMessageBrowse', array('id' => $item->getForumId())).'">'.$Lang->getText('Forum.ApproveMessages').'</a>';
				} else {
					$trow[]= '&nbsp;';
				}
				$trow[]= '<a href="'.GFUrl::getUrl()->currentBaseUrl('ForumDelete', array('id' => $item->getForumId())).'">'.$Lang->getText('Forum.DeletePageName').'</a>';
		
				$ret .= $theme->tableRow($trow, array(), false);
			}
		}
	
		$ret .= $theme->tableBottom(array(GFUrl::getUrl()->currentBaseUrl('ForumAdd')=>$Lang->getText('Forum.AddNew')));
		return $ret;
	}
	
	private function postAddNew($redirect_to) {
		try {
			$Forum = new Forum();
			$Forum->setSection($this->section);
			$Forum->setRefId($this->ref_id);	
			$Forum->setForumName( Validator::sanitizeString($_POST['forum_name']) );
			$Forum->setDescription( Validator::sanitizeString($_POST['description']) );
			$Forum->setSendAllPostsTo( Validator::sanitizeString($_POST['send_all_posts_to']) );
			$Forum->setModerationLevel( (isset($_POST['moderation_level']) ? Validator::sanitizeInt($_POST['moderation_level']) : 0) );
			$Forum->setShowAssociationsTab( isset($_POST['show_associations_tab']) ? true : false );
			$Forum->setShowTagsTab( isset($_POST['show_tags_tab']) ? true : false );
		
			$Forum->save();
		} catch (InvalidDataException $ex) {
			$this->errors = $ex->getErrors();
			return $this->addNewForumForm($redirect_to, true);
		}
		
		// redirect on success
		GFUrl::redirect($redirect_to, array('feedback' => $this->language->getText('Forum.AddFeedback')));
	}
	
	/**
	 * Returns true if the form was processed and errors were found
	 * @return bool
	 */
	public function hasErrors() {
		return (count($this->errors) > 0);
	}
	
	/**
	 * Returns an array with the errors found after processing the POST data
	 * @return array
	 */
	public function getErrors() {
		return $this->errors;
	}

	/**
	 * @return string
	 * @param	int		$forum_id		ID of the forum to edit
	 * @param	string	$redirect_to	URL to redirect on succesful add
	 * @param	bool	$force_form		Whether to force the display of the form (disables processing the POST data)
	 */
	public function editForumForm($forum_id, $redirect_to, $force_form=false) {
		$Forum = ForumPeer::retrieveByPk($forum_id);
		if (!is_object($Forum)) {
			throw new PermissionDeniedException();
		} else if ($Forum->getSection() != $this->section || $Forum->getRefId() != $this->ref_id) {
			throw new PermissionDeniedException();
		}

		// See if we've submitted the form. In that case, process the data.
		if (isset($_POST['forum_edit_submit']) && !$force_form) {
			return $this->postEdit($Forum, $redirect_to);
		}
		
		$formFile = SystemUtils::buildFilePath($this->wwwlibPath, '/ForumEdit-tpl.php');
		$forum_name = ( (isset($_POST) && isset($_POST['forum_name'])) ? htmlspecialchars($_POST['forum_name']) : $Forum->getForumName() );
		$description = ( (isset($_POST) && isset($_POST['description'])) ? htmlspecialchars($_POST['description']) : $Forum->getDescription() );
		$send_all_posts_to = ( (isset($_POST) && isset($_POST['send_all_posts_to'])) ? htmlspecialchars($_POST['send_all_posts_to']) : $Forum->getSendAllPostsTo() );
		$moderation_level = ( (isset($_POST) && isset($_POST['moderation_level'])) ? htmlspecialchars($_POST['moderation_level']) : $Forum->getModerationLevel() );
		if (isset($_POST['forum_edit_submit'])) {
			$show_associations_tab = (isset($_POST['show_associations_tab']) ? true : false);
			$show_tags_tab = (isset($_POST['show_tags_tab']) ? true : false);
		} else {
			$show_associations_tab = $Forum->getShowAssociationsTab();
			$show_tags_tab = $Forum->getShowTagsTab();
		}
		
		$theme = $this->template;
		$Lang = $this->language;

		ob_start();	
		include($formFile);
		$form = ob_get_clean();
		
		return $form;
	}
	
	private function postEdit(Forum $forum, $redirect_to) {
		try {
			$forum->setForumName( Validator::sanitizeString($_POST['forum_name']) );
			$forum->setDescription( Validator::sanitizeString($_POST['description']) );
			$forum->setSendAllPostsTo( Validator::sanitizeString($_POST['send_all_posts_to']) );
			$forum->setModerationLevel( Validator::sanitizeInt($_POST['moderation_level']) );
			$forum->setShowAssociationsTab( isset($_POST['show_associations_tab']) ? true : false );
			$forum->setShowTagsTab( isset($_POST['show_tags_tab']) ? true : false );
			
			$forum->save();
		} catch (InvalidDataException $ex) {
			$this->errors = $ex->getErrors();
			return $this->editForumForm($forum->getForumId(), $redirect_to, true);
		}

		// redirect on success
		$Project = GFUrl::getUrl()->getProject();
		$perm = $Project->getPermission(); 
		if ($perm->check(Forum::SECTION, 0, Forum::ADMIN)) {
			GFUrl::redirect($redirect_to, array('feedback' => $this->language->getText('Forum.EditFeedback')));
		} else {
			GFUrl::redirect(GFUrl::getUrl()->projectURL($Project, '','','forum',array('feedback'=>$this->language->getText('Forum.EditFeedback'))));
		}
	}
	
	/**
	 * @return string
	 * @param	int		$forum_id		ID of the forum to delete
	 * @param	string	$redirect_to	URL to redirect on succesful delete
	 * @param	bool	$force_form		Whether to force the display of the form (disables processing the POST data)
	 */
	public function deleteForumForm($forum_id, $redirect_to, $force_form=false) {
		$Forum = ForumPeer::retrieveByPk($forum_id);
		if (!is_object($Forum)) {
			throw new PermissionDeniedException();
		} else if ($Forum->getSection() != $this->section || $Forum->getRefId() != $this->ref_id) {
			throw new PermissionDeniedException();
		}

		// See if we've submitted the form. In that case, process the data.
		if (isset($_POST['forum_delete_submit']) && !$force_form) {
			return $this->postDelete($Forum, $redirect_to);
		}
		
		$formFile = SystemUtils::buildFilePath($this->wwwlibPath, '/ForumDelete-tpl.php');
		
		$theme = $this->template;
		$Lang = $this->language;

		ob_start();	
		include($formFile);
		$form = ob_get_clean();
		
		return $form;
	}
	
	private function postDelete(Forum $forum, $redirect_to) {
		if (isset($_POST['sure'])) {
			$forum->delete();
		} else {
			$this->errors = array($this->language->getText('ForumDeleteForm.DidntConfirm'));
			return $this->deleteForumForm($forum->getForumId(), $redirect_to, true);
		}
		GFUrl::redirect($redirect_to, array('feedback' => $this->language->getText('Forum.DeleteFeedback')));
	}
}

?>
