<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version $Id$
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once('lib/plugin/GFPlugin.php');
require_once('plugin/ActionManager.php');
require_once('SystemUtils.php');
require_once('plugins/frs/lib/FrsPackagePeer.php');
require_once('plugins/frs/lib/FrsPackage.php');
require_once('plugins/frs/lib/FrsReleasePeer.php');
require_once('lib/core/ProjectPluginPeer.php');
require_once('lib/core/PluginPeer.php');
require_once('lib/GFExportImportUtils.php');
require_once('plugins/frs/lib/FrsProjectSetting.php');

class FrsPlugin implements GFProjectPlugin {

	protected $project;
	const READ = 0;
	const WRITE = 1;
	const ISPRIVATE = 0;
	const ISPUBLIC = 1;	
	const SECTION = 'frs';
	
	function __construct() {
	}

	/**
	 *	this is called when the first Action is called. Action manager 
	 *	gets the listeners for each plugin.
	 */
	function initialize() {
		$listenTo = array(
			'BUILD_PROJECT_TABS',
			'BUILD_PROJECT_FRS_SUMMARY',
			'GET_PROJECT_ADMIN_NAV_ELEMENTS',
			'GET_PROJECT_REPORTING_NAV_ELEMENTS',
			'SEARCH_BY_ID_LIST',
			'SEARCH_BY_ID_ACTION',
			'BUILD_SEARCH_COMBO',
			'DO_SEARCH',
			'PROJECT_APPROVE',
			'BUILD_MYPAGE_ELEMENTS',
			'BUILD_MYPAGE_MONITORS',
			'CHECK_DOWNLOAD_PERMS',
			'MONITOR_CHECK_PERMISSIONS',
			'GET_ROLE_CHOICES',
			'UPDATE_ROLE_CHOICES',
			'GET_OBSERVER_ROLE_CHOICES',
			'UPDATE_OBSERVER_SETTINGS',
			'RENDER_ASSOCIATION_LINK',
			'GET_ASSOCIATED_ITEM_INFO',
			'ASSOCIATION_SECTION',
			'ASSOCIATION_CHECK',
			'GET_DISK_USAGE',
			'PROJECT_FULLEXPORT',
			'PROJECT_FULLIMPORT',
			'PROJECT_TPLEXPORT',
			'PROJECT_TPLIMPORT',
			'SCM_COMMIT',
			'PROJECT_AUDIT_TRAIL',
			'USER_PROJECT_DELETE',
			'USERGROUP_USER_REMOVE',
			'GET_PROJECT_SETTINGS_FORM',
			'SAVE_PROJECT_SETTINGS',
			'DO_TAG_SEARCH'
	//		'GET_FRS_NAV_ELEMENTS'
			);
		ActionManager::registerListeners($this, $listenTo);
	}

	
	/**
	 * returns the search results for the given input
	 *
	 * @param array $params
	 * @return resultset
	 */
	private function checkDownloadPerms($params) {
		$section = $params['section'];
		$ref_id = $params['ref_id'];
		$result = new PluginResult($this);
		$result->setResult(false); // false by default, looking for true
		// if it's for another section, exit
		if ($section != FrsRelease::SECTION ) {
			$result->setResult(false);
		} else {
			$release = FrsReleasePeer::retrieveByPK($ref_id);
			
			if (is_object($release)) {
				$package = $release->getFrsPackage();
				// if the package requires login and not logged in, send to login page (this should have been filtered before in frs/wwwlib but just in case)
				if (($package->getRequireLogin()) && (!Session::getSession()->isLoggedIn())) {
					$result->setResult(false);
					return $result;
				}
				// public project with public package can download
				if ($package->userCanRead()) {
					$result->setResult(true);
				} else {					
					$result->setResult(false);
				}
			}
		}
		return $result;
	}	
	
	/**
	 *	Different types of plugins are possible -
	 *	GFPlugin::SITE_PLUGIN = 'site'
	 *	GFPlugin::PROJECT_PLUGIN = 'project'
	 *	GFPlugin::USER_PLUGIN = 'user'
	 */
	function getType() {
		return GFPlugin::PROJECT_PLUGIN;
	}

	/**
	 *	get the name of this plugin. example: 'tracker'
	 *	@return	string
	 */
	function getName() {
		return 'frs';
	}

	/**
	 *	get the array of action names that this plugin answers to.
	 *	@return	array
	 */
	function getListeners() {
		return array(); //XXX
	}

	/**
	 * returns the part of the html text that goes into the my page area, for monitored items
	 *
	 * @param array $params needed params to build the html
	 * @return string	the builded html result
	 */
	private function buildMyPageMonitors($params) {	
		$con = Propel::getConnection(ForumPeer::DATABASE_NAME);
		$theme = $params['theme'];

		//get the assigned project forums for the user
		$user = Session::getSession()->getUser();
		$user_id = $user->getPrimaryKey();
		$sql = "select f.frs_package_id, f.package_name, p.project_id,p.project_name from frs_package f, monitor m, project p
		where m.user_id = '$user_id' and m.section='" . FrsPackage::SECTION . "' and m.ref_id = f.frs_package_id and p.project_id = f.project_id";
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);

		$value = '<h3>'.Language::getSessText('Frs.Monitoring').'</h3>';

		$theader = array();
		$theader[] = 'Project';
		$theader[] = Language::getSessText('My.Packages');
		
		$last_project = null;
		
		if ($rs->getRecordCount()>0) {
			$theader[] = '';
			$value .= $theme->tableTop($theader);
			$ttype=array();
			while($rs->next()) {
				$ttype[$rs->getInt('project_id')][] = '1';
			}			
			$rs->beforeFirst();
			while($rs->next()) {
				$trow = array();
				$tmod = array();
				$package_name = $rs->getString('package_name');
				$project_name = $rs->getString('project_name');
				if ($last_project != $rs->getInt('project_id')) {
					$trow[] = '<a href="' . GFUrl::getUrl()->projectURL(ProjectPeer::retrieveByPk($rs->getInt('project_id')),'FrsReleaseBrowse',$this->getName(),'',array('frs_package_id'=>$rs->getInt('frs_package_id'))) . '" >' .  $project_name . '</a>';
					$tmod[]='rowspan="'.count($ttype[$rs->getInt('project_id')]).'" width="30%"';
				}
				$last_project = $rs->getString('project_id');
				$trow[] = '<a href="' . GFUrl::getUrl()->projectURL(ProjectPeer::retrieveByPk($rs->getInt('project_id')),'FrsReleaseBrowse',$this->getName(),'',array('frs_package_id'=>$rs->getInt('frs_package_id'))) . '" >' .  $package_name . '</a>';
				$tmod[] = '';
				$trow[] = '<a href="'.GFUrl::getUrl()->getMonitorUrl(FrsPackage::SECTION, $rs->getInt('frs_package_id'), null, false).'" >'.Language::getSessText('TrackerItem.StopMonitor').'</a>';
				$tmod[] = 'align="right"';
				$value .= $theme->tableRow($trow, $tmod);
			}
		} else {
			$value .= $theme->tableTop($theader);
			$trow = array();
			$tmod[]='colspan="2"';
			$trow[] = Language::getSessText('My.NoMonitoredPackages');
			$value .= $theme->tableRow($trow,$tmod);
		}
		$value .= $theme->tableBottom();
		
		$result = new PluginResult($this);
		$result->setResult($value);
		return $result;		
	}		
	
	/**
	 *	handle action has a case statement to redirect the action to the appropriate function.
	 *	@param string
	 *	@param mixed

	 *	@return PluginResult
	 */
	function handleAction($listenerName,$params) {
		switch ($listenerName) {
			case 'BUILD_PROJECT_TABS': {
				return $this->_buildMainTabsListener($params);
				break;
			}
			case 'BUILD_PROJECT_FRS_SUMMARY': {
				return $this->_buildProjectSummary($params);
				break;
			}
			case 'GET_PROJECT_ADMIN_NAV_ELEMENTS': {
				return $this->getProjectAdminNavElements($params);
				break;
			}
			case 'GET_PROJECT_REPORTING_NAV_ELEMENTS': {
				return $this->getProjectReportingNavElements($params);
				break;
			}
			case 'SEARCH_BY_ID_LIST': {
				return $this->searchByIDList();
				break;
			}
			case 'SEARCH_BY_ID_ACTION': {
				return $this->searchByIDAction($params);
				break;
			}
			case 'DO_SEARCH': {
				return $this->getSearchResults($params);
				break;
			}	
			case 'BUILD_SEARCH_COMBO': {
				return $this->buildSearchOptions($params);
				break;
			}		
			case 'PROJECT_APPROVE': {
				return $this->approveActions($params);
				break;
			}				
			case 'BUILD_MYPAGE_ELEMENTS': {
				return $this->buildMyPageElements($params);
				break;
			}
			case 'BUILD_MYPAGE_MONITORS': {
				return $this->buildMyPageMonitors($params);
				break;
			}		
			case 'CHECK_DOWNLOAD_PERMS': {
				return $this->checkDownloadPerms($params);
				break;
			}
			case 'MONITOR_CHECK_PERMISSIONS': {
				return $this->monitorCheckPermissions($params);
				break;
			}
			case 'GET_ROLE_CHOICES': {
				return $this->getRoleChoices($params);
				break;
			}
			case 'UPDATE_ROLE_CHOICES': {
				return $this->updateRoleChoices($params);
				break;
			}
			case 'GET_OBSERVER_ROLE_CHOICES': {
				return $this->getObserverRoleChoices($params);
				break;
			}
			case 'UPDATE_OBSERVER_SETTINGS': {
				return $this->updateObserverSettings($params);
				break;
			}
			case 'RENDER_ASSOCIATION_LINK': {
				return $this->renderAssociationLink($params);
				break;
			}
			case 'GET_ASSOCIATED_ITEM_INFO': {
				return $this->getAssociatedItemInfo($params);
				break;
			}
			case 'ASSOCIATION_SECTION': {
				return $this->getAssociationSection($params);
				break;
			}
			case 'ASSOCIATION_CHECK': {
				return $this->getAssociationCheck($params);
				break;
			}
			case 'GET_DISK_USAGE': {
				return $this->getDiskUsage($params);
				break;
			}
			case 'PROJECT_FULLIMPORT':
			case 'PROJECT_FULLEXPORT': {
				return GFExportImportUtils::getSpecData($this);
				break;
			}
			case 'PROJECT_TPLIMPORT':
			case 'PROJECT_TPLEXPORT': {
				return GFExportImportUtils::getSpecData($this, 'specDataTpl.xml');
				break;
			}
			case 'PROJECT_AUDIT_TRAIL': {
				return $this->getProjectAuditTrail($params);
				break;
			}
			case 'SCM_COMMIT': {
				return $this->scmCommit($params);
				break;
			}
			case 'USER_PROJECT_DELETE': {
				return $this->deleteUserMonitors($params);
				break;
			}
			case 'USERGROUP_USER_REMOVE': {
				return $this->deleteUsergroupUserMonitors($params);
				break;
			}
			case 'GET_PROJECT_SETTINGS_FORM': {
				return $this->getProjectSettingsForm($params);
				break;
			}
			case 'SAVE_PROJECT_SETTINGS': {
				return $this->saveProjectSettings($params);
				break;
			}
			case 'DO_TAG_SEARCH': {
				return $this->getSearchByTagsResult($params);
				break;
			}
		}
	}


	private function deleteUsergroupUserMonitors($usergroup_user) {
		$con = Propel::getConnection();		

		$project_ids = $usergroup_user->getProjectMonitorsToDelete();
		$user_id = $usergroup_user->getMemberUserId();

		if ($project_ids != '') {
			$stmt = $con->createStatement();
			//Delete monitor for the projects
			$sql = "DELETE FROM monitor WHERE
							section='".FrsPackage::SECTION."'
							AND ref_id IN 
								(
									SELECT frs_package_id FROM frs_package, project WHERE
								 	frs_package.project_id = project.project_id
								 	AND
								 	project.project_id IN (".$project_ids.")
								 	AND
								 	NOT (project.is_public = true AND frs_package.is_public = true)
								)
							AND user_id = ".$user_id;

			$stmt->executeQuery($sql);
		}		
	}

							

	private function deleteUserMonitors($user_project) {
		$user_id = $user_project->getUserId();
		$project = $user_project->getProject();
		if (is_object($project)) {
			if (!$project->isPublic()) {
				$con = Propel::getConnection();
				$stmt = $con->createStatement();
		
				$sql = "DELETE FROM monitor WHERE section='".FrsPlugin::SECTION."' AND ref_id = ".$project->getProjectId()."
								AND user_id = ".$user_id;
		
				$stmt->executeQuery($sql);
			}
		}
	}

	private function getAssociationCheck($params) {
		if ($params->getSection() == FrsRelease::SECTION) {
			$release=FrsReleasePeer::retrieveByPk($params->getRefId());
			if (!is_object($release) || !$release->userCanUpdate()) {
				throw new InvalidDataException(array('The release doesn\'t exist or you don\'t have permissions for it'));
			}
		} 
		if ($params->getToSection() == FrsRelease::SECTION) {
			$release=FrsReleasePeer::retrieveByPk($params->getToRefId());
			if (!is_object($release) || !$release->userCanUpdate()) {
				throw new InvalidDataException(array('The release item doesn\'t exist or you don\'t have permissions for it'));
			}
		}
	}

	private function getAssociationSection($params) {
		if (!is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}
		$result = new PluginResult($this);
		$result->setResult(array(FrsRelease::SECTION,'Release'));
		return $result;
	}

	private function renderAssociationLink($params) {
		$ret=array();
		foreach ($params as $param) {
			if ($param[0]==FrsRelease::SECTION) {
				$ti=FrsReleasePeer::retrieveByPk($param[1]);
				if (is_object($ti)) {
					$ret[]=array('Release: '.$ti->getFrsReleaseLink(true),
					$param[2],
					$param[3]);
				}
			}
		}
		$result = new PluginResult($this);
		$result->setResult($ret);
		return $result;
	}
	
	/**
	 *	Returns additional info for the associated item
	 *		 
	 *	@param array $params needed params to build the associated item info
	 * 	@return array with associated item info (Description, URL)
	 */	 
	private function getAssociatedItemInfo($params) {
		$ret=array();
		foreach ($params as $param) {
			if ($param[0]==FrsRelease::SECTION) {
				$ti=FrsReleasePeer::retrieveByPk($param[1]);
				if (is_object($ti)) {
					$ret[]=array(
					$param[2],
					$param[3],
					$param[4],
					$param[5],
					$param[6],
					'Release: '.$ti->getReleaseName(),
					$ti->getFrsReleaseLink(false));
				}
			}
		}
		$result = new PluginResult($this);
		$result->setResult($ret);
		return $result;
	}
	
	/**
	 * returns the part of the html text that goes into the my page area
	 *
	 * @param array $params needed params to build the html
	 * @return string	the builded html result
	 */
	private function buildMyPageElements($params) {	
		
	}

	/**
	 * returns the options to print in the combobox
	 *
	 * @param unknown_type $params
	 * @return unknown
	 */
	private function buildSearchOptions($params) {
		$result = new PluginResult($this);
		
		if (isset($params['project'])) {
			if (!$params['project']->usesPlugin($this->getName())) {
				return;
			}
			$text = Language::getSessText('Frs.ThisProjectFrs');
		} else {
			$text = Language::getSessText('Frs.AllProjectsFrs');
		}
		
		$value = array();
		// only give result if we're in a project
		$value[] = array(
			'value'	=> $this->getName(),
			'display'	=> $text
			);
		
		$result->setResult($value);
		return $result;		
	}
	
	/**
	 * returns the search results for the given input
	 *
	 * @param array $params
	 * @return resultset
	 */
	private function getSearchResults($params) {
		if (array_search($this->getName(), $params['search']) !== false) {
			$result = new PluginResult($this);
			$searcher = new GFSearch($params);
			$result->setResult($this->search($searcher));
			return $result;
		}
	}


	private function getSearchByTagsResult($params) {
		
		$result = new PluginResult($this);
		$search = new GFSearch($params);

		$result->setResult($this->searchByTags($search));

		return $result;

	}

	/**
	 *	get project uri to be used as root for constructing uris
	 *	@return string
	 */
	function getProjectPluginUri() {
		return GFUrl::getUrl()->projectURL($this->project,'','frs');
	}

	/**
	 *      return an sql statement for calculating disk usage
	 */
	function getDiskUsage($params) {
		$sql="select q.project_id,sum(q.usage) as usage FROM (
		--frs releases
		SELECT frsp.project_id,sum((length(frsr.release_notes)*2)+(length(frsr.changes)*2)) AS usage
			FROM frs_package frsp JOIN frs_release frsr USING (frs_package_id)
			GROUP BY project_id
		UNION
		--released files
		SELECT frsp.project_id,sum(fs.file_size+length(fsi.vectors)) AS usage
			FROM frs_package frsp JOIN frs_release frsr USING (frs_package_id),
			filesystem fs JOIN filesystem_idx fsi USING (filesystem_id)
			WHERE frsr.frs_release_id=fs.ref_id
			AND fs.section='frsrelease'
			GROUP BY project_id
		) q
		GROUP BY project_id";
		$result = new PluginResult($this);
		$result->setResult($sql);
		return $result;
	}

	/**
	 *	take a search object and handle the search internally.
	 *	@param	GFSearch
	 *	@return PluginResult
	 */
	function search(GFSearch $search) {
		if (!$search->getProjectId()) {
			return $this->globalSearch($search);
		}

		if (GFConf::get('system.dbtype') == 'postgres') {
			$con = Propel::getConnection(UserPeer::DATABASE_NAME);
			$words = $search->getWordsForQuery();

			if ($search->getStartDate()) {
				$date_sql = " AND frsr.release_date BETWEEN '".$search->getStartDate()."' AND '".$search->getEndDate()."' ";
			} else {
				$date_sql='';
			}

			if ($search->getSearchAttach()) {
				$attach_sql="UNION
				SELECT frsr.frs_release_id, rank(filesystem_idx.vectors, q) AS rank
				FROM filesystem_idx, to_tsquery('".$words."') q, filesystem fs, frs_release frsr, frs_package frsp
				WHERE filesystem_idx.vectors @@ q
				AND filesystem_idx.filesystem_id=fs.filesystem_id
				AND fs.section='frsrelease'
				AND fs.ref_id=frsr.frs_release_id
				AND frsr.frs_package_id=frsp.frs_package_id
				$date_sql
				AND frsp.project_id='".$search->getProjectId()."'";
			} else {
				$attach_sql='';
			}

			$sql = "SELECT frs_release.frs_release_id,frs_release.release_name,frs_release.release_date,sum(s.rank) as rank
			FROM frs_release, (
			SELECT fri.frs_release_id, rank(fri.vectors, q) AS rank
			FROM
			frs_release_idx fri, frs_release frsr, frs_package frsp, to_tsquery('".$words."') q
			WHERE fri.vectors @@ q 
			AND fri.frs_release_id=frsr.frs_release_id
			AND frsp.project_id = '" . $search->getProjectId() . "'
			AND frsp.frs_package_id = frsr.frs_package_id 
			$date_sql
			$attach_sql
			) s
			WHERE s.frs_release_id=frs_release.frs_release_id
			GROUP BY frs_release.frs_release_id,release_name,release_date
			ORDER BY rank DESC";
			//echo $sql;

			$stmt = $con->createStatement();
			$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);
			$template = $search->getTemplate();
			if ($rs->getRecordCount()>0) {
				$ret = '';
				$current_package = '0'; // to see if the package has changed
				$project = ProjectPeer::retrieveByPk($search->getProjectId());
				$ret = '<p><h3>'.Language::getSessText('Frs.SearchResultsFor').'</h3></p>';
				$ret .= $template->tableTop(array(Language::getSessText('FrsRelease.ReleaseNameName'),Language::getSessText('FrsRelease.ReleaseDateName')));
				$i = 0;
				while ($rs->next()) {
					$relid = $rs->getInt('frs_release_id');
					$trow = array();
					$trow[] = '<a href="'.GFUrl::getUrl()->projectURL($project,'FrsReleaseView',$this->getName(),'',array('frs_package_id'=>$current_package,'release_id'=>$relid)) . '">' . $rs->getString('release_name').'</a>';
					$trow[] = $rs->getDate('release_date');
					$ret .= $template->tableRow($trow);
				}
				$ret .= $template->tableBottom();
				return $ret;
			} else {
				// cero results for search
				return '<p><h3>'.Language::getSessText('Frs.SearchResultsFor').'</h3></p>' . $template->tableTop(array(Language::getSessText('FrsRelease.ReleaseNameName'),Language::getSessText('FrsRelease.ReleaseDateName'))) . $template->tableFullRow( Language::getSessText('Search.NoResults') ) . $template->tableBottom();
			}
		} elseif (GFConf::get('system.dbtype') == 'mysql') {
			
		} elseif (GFConf::get('system.dbtype') == 'oracle') {
			
		}		
	}

	/**
	 *	Searches across all projects
	 *	@param	GFSearch
	 *	@return string
	 */
	private function globalSearch($search) {
		if (GFConf::get('system.dbtype') == 'postgres') {
			$con = Propel::getConnection(UserPeer::DATABASE_NAME);
			$words = $search->getWordsForQuery();

			if ($search->getStartDate()) {
				$date_sql = " AND frsr.release_date BETWEEN '".$search->getStartDate()."' AND '".$search->getEndDate()."' ";
			} else {
				$date_sql='';
			}

			if ($search->getSearchAttach()) {
				$attach_sql="UNION
				SELECT frsr.frs_release_id, rank(filesystem_idx.vectors, q) AS rank,
				headline(frsr.release_name, q) AS headline_release_name
				FROM filesystem_idx, to_tsquery('".$words."') q, filesystem fs, frs_release frsr, frs_package frsp
				WHERE filesystem_idx.vectors @@ q
				AND filesystem_idx.filesystem_id=fs.filesystem_id
				AND fs.section='frsrelease'
				AND fs.ref_id=frsr.frs_release_id
				AND frsr.frs_package_id=frsp.frs_package_id
				$date_sql";
			} else {
				$attach_sql='';
			}

			$sql = "SELECT 
				project.project_name, 		-- 1
				project.unix_name, 			-- 2
				sum(s.rank) as rank,		-- 3
				s.headline_release_name,	-- 4
				frs_release.*				-- 5
			FROM frs_release, frs_package, (
				SELECT fri.frs_release_id, rank(fri.vectors, q) AS rank,
				headline(frsr.release_name, q) AS headline_release_name
				FROM
				frs_release_idx fri, frs_release frsr, frs_package frsp, to_tsquery('".$words."') q
				WHERE fri.vectors @@ q 
				AND fri.frs_release_id=frsr.frs_release_id
				AND frsp.frs_package_id = frsr.frs_package_id 
				$date_sql
				$attach_sql
			) s, project
			WHERE s.frs_release_id=frs_release.frs_release_id AND frs_release.frs_package_id=frs_package.frs_package_id
			AND frs_package.project_id=project.project_id
			GROUP BY project.project_name, project.unix_name, frs_release.frs_release_id,frs_release.frs_package_id,
			frs_release.release_name,frs_release.release_notes,frs_release.changes,frs_release.status_id,
			frs_release.preformatted,frs_release.release_date,frs_release.released_by,frs_release.is_released,s.headline_release_name
			ORDER BY rank DESC";
			//echo $sql;

			$stmt = $con->createStatement();
			$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);
			$template = $search->getTemplate();

			$count = 0;
			$ret = '';
			$project = ProjectPeer::retrieveByPk($search->getProjectId());
			$ret = '<p><h3>'.Language::getSessText('Frs.SearchResultsFor').'</h3></p>';
			$ret .= $template->tableTop(array(
					Language::getSessText('Project.ListNameTitle'),
					Language::getSessText('FrsRelease.ReleaseNameName'),
					Language::getSessText('FrsRelease.ReleaseDateName')
				));
			while ($rs->next()) {
				$release = new FrsRelease();
				$release->hydrate($rs, 5);
				
				if (!$release->userCanRead()) {
					continue;
				}
				
				$count++;
				$project_unix_name = $rs->getString(2);
				$relid = $release->getFrsReleaseId();
				$trow = array();
				$trow[] = '<a href="'.GFUrl::getUrl()->projectUrl($project_unix_name).'">'.$rs->getString(1).'</a>';
				$trow[] = '<a href="'.GFUrl::getUrl()->projectURL($project_unix_name,'FrsReleaseView',$this->getName(),'',array('frs_package_id'=>$release->getFrsPackageId(),'release_id'=>$relid)) . '">' . $rs->getString(4).'</a>';
				$trow[] = $release->getReleaseDate();
				$ret .= $template->tableRow($trow);
			}
			$ret .= $template->tableBottom();
			
			
			if ($count > 0) {
				return $ret;
			} else {
				// cero results for search
				return '<p><h3>'.Language::getSessText('Frs.SearchResultsFor').'</h3></p>' . $template->tableTop(array(Language::getSessText('FrsRelease.ReleaseNameName'),Language::getSessText('FrsRelease.ReleaseDateName'))) . $template->tableFullRow( Language::getSessText('Search.NoResults') ) . $template->tableBottom();
			}
		} elseif (GFConf::get('system.dbtype') == 'mysql') {
			
		} elseif (GFConf::get('system.dbtype') == 'oracle') {
			
		}
	}


	function searchByTags(GFSearch $search){

		$words = $search->getWordsForQuery();
		$project_id = $search->getProjectId();
		$template = $search->getTemplate();

		//Clean key words from tsearch operators
		$words = preg_replace('/,+/',',', $words);
		$tsearch_operators = array('(', ')', '!', '|','&');
		$replacements = array('','','','',',');
		$words = str_replace($tsearch_operators, $replacements, $words);
		$words_arr = explode(',', $words);

		$wc = count($words_arr);
		if($wc == 0){
			return '';
		}

		for($i = 0; $i < $wc; $i++){
			$words_arr[$i] = "'" . $words_arr[$i] . "'";
		}
		$words = implode(',', $words_arr);

		if($project_id) {
			$frsrelease_prj = " AND frs_package.project_id = $project_id ";
		} else {
			$frsrelease_prj = ' ';
		}

		$frs_release_perms_sql = "SELECT DISTINCT frs_package.frs_package_id AS frs_package_id
		FROM frs_package INNER JOIN frs_release ON frs_package.frs_package_id = frs_release.frs_package_id
		WHERE frs_release.frs_release_id IN
		(SELECT tag_entry.ref_id FROM tag_entry WHERE
		section='frsrelease'
		AND tag_name IN ($words)
		GROUP BY tag_entry.ref_id
		HAVING count(tag_entry.ref_id) = $wc) $frsrelease_prj";

		$con = Propel::getConnection();
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($frs_release_perms_sql, ResultSet::FETCHMODE_NUM);

		$frsrelease_aux = null;
		if($rs->getRecordCount() > 0) {
			while($rs->next()){
				$frs_packages[] = $rs->getInt(1);
			}
			$frsps = FrsPackagePeer::retrieveByPks($frs_packages);

			$frs_packages= array();
			foreach($frsps as $frsp){
				$setting = FrsProjectSettingPeer::retrieveByPk($frsp->getProject()->getProjectId());
				if($frsp->getProject()->usesPlugin('frs') 
					&& $frsp->userCanRead() &&
					(!$setting || $setting->getShowTagsTab())
					){
					$frs_packages[] = $frsp->getFrsPackageId();
				}
			}
			if(count($frs_packages) == 0){
				//no results
				return null;
			}else {
				$frsrelease_aux = ' AND frs_package.frs_package_id IN ( '. implode(',', $frs_packages) .')';
			}
		} else {
			//empty result set
			return null;
		}

		$frs_release_sql = "SELECT 'frsrelease' AS section, frs_release.frs_release_id as ref_id,
		frs_release.release_name AS description, project.unix_name AS prj_unix_name,
		project.project_name AS prj_name,
		frs_package.frs_package_id AS frs_package_id,
		frs_package.package_name AS package_name
		FROM frs_release INNER JOIN frs_package ON frs_release.frs_package_id = frs_package.frs_package_id
		INNER JOIN project ON frs_package.project_id = project.project_id
		WHERE frs_release.frs_release_id IN
		(SELECT tag_entry.ref_id FROM tag_entry WHERE section='frsrelease'
		AND tag_name IN ($words)
		GROUP BY tag_entry.ref_id
		HAVING COUNT(tag_entry.ref_id) = $wc) $frsrelease_aux $frsrelease_prj";

		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($frs_release_sql, ResultSet::FETCHMODE_ASSOC);


		$ret = null;

		if($rs->getRecordCount() > 0){
			$Lang = Session::getSession()->getLanguage();
			while($rs->next()){
				$trow = array();
				if(!$project_id){
					$trow[] = '<a href="' . GFUrl::getUrl()->projectUrl($rs->getString('prj_unix_name')) . '">'. $rs->getString('prj_name') . '</a>';
				}

				$trow[] = $rs->getInt('ref_id');
				$trow[] = $Lang->getText('Tag.FrsReleaseSection');
				$trow[] = '<a href="'. GFUrl::getUrl()->projectUrl($rs->getString('prj_unix_name'), 'FrsReleaseBrowse','frs','',array('frs_package_id'=>$rs->getString('frs_package_id'))) . '">' . $rs->getString('package_name') . '</a>';
				$trow[] = '<a href="'. GFUrl::getUrl()->projectUrl($rs->getString('prj_unix_name'), 'FrsReleaseView', 'frs', '', array('release_id'=>$rs->getInt('ref_id'))) . '">' . $rs->getString('description') . '</a';
				$ret .= $template->tableRow($trow);
			}
		}

		return $ret;

	}

	/**
	 *      return resultset of an array to populate the ID search select box
	 *      @return PluginResult
	 */
	function searchByIDList() {
		$pr=new PluginResult($this);
		$pr->setResult(array(FrsRelease::SECTION => 'FRS Release'));
		return $pr;
	}
	/**
	 *      return result set of URL to redirect to
	 *      @param $params array
	 *      @return PluginResult
	 */
	function searchByIDAction($params) {
		if (isset($params['searchtype']) && $params['searchtype'] == FrsRelease::SECTION) {
			$pr=new PluginResult($this);
			$ti=FrsReleasePeer::retrieveByPk($params['searchid']);
			if (!is_object($ti)) {

			} else {
				$pr->setResult($ti->getFrsReleaseLink());
			}
			return $pr;
		}
	}
	/**
	 * 	The addMember listener - this hook will fire
	 *	whenever a member is added to a project.
	 *	@param	User	User object
	 *	@return	boolean
	 */
	function addMemberListener($params) {
		return false; // XXX
	}

	/**
	 * 	The removeMember listener - this hook will fire
	 *	whenever a member is removed from a project.
	 *	@param	User	User object
	 *	@return	boolean
	 */
	function removeMemberListener($params) {
		return false; // XXX
	}

	/**
	 * 	The approveProject listener - this hook will fire
	 *	when a project is approved.
	 *	@param	Project	the project object
	 *	@return	boolean
	 */
	function approveProjectListener($params) {
		return false; // XXX
	}

	/**
	 * 	The deleteProject listener - this hook will fire
	 *	when a project is deleted.
	 *	@return	boolean
	 */
	function deleteProjectListener() {
		return false; // XXX
	}

	/**
	 * 	The checkPermission - this listener will fire
	 *	when permissions are being checked for this project.
	 *	@param	Array	An associative array containing 
	 *	'section_name' a name like 'tracker' or 'tracker_item'
	 *	'section_id'=>id (the id of the section, for example a specific tracker)
	 *	'subsection_id'=>id (the id of the subsection, for example an tracker_item id)
	 *	'perm_level'=>int (the value, such a 1 for read, 2 for write, that is being requested)
	 *	@return	boolean
	 * /
	function checkPermission($params);
*/
	
	private function _buildMainTabsListener($params) {
		if (!isset($params['project']) || !is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}
		$project = $params['project'];

		$result = new PluginResult($this);
		$value = array(
				'text'	=> Language::getSessText('Frs.TabString'),
				'action'	=> GFUrl::getUrl()->projectURL($project,'','frs'),
				'id'	=> 'frs'
			);
		$result->setResult($value);
		return $result;
	}

	/**
	 *      getFrsNavElements - get the local nav elements for the tracker
	 *      @return PluginResult
	 * /
	private function getFrsNavElements() {
		$result = new PluginResult($this);
		$arr = array(
			'title'=>'FRS Title',
			'elements'=> array(
			array('link'=>'/path/to/file/?action',
				'description'=>'Title 1'),
			array('link'=>'/path/to/file/?action',
				'description'=>'Title 2'),
			array('link'=>'/path/to/file/?action',
				'description'=>'Title 3')
			)
		);
		$result->setResult($arr);
		return $result;
	}

	/**
	 *      getProjectAdminNavElements - get the local nav elements for the tracker
	 *      @return PluginResult
	 */
	private function getProjectAdminNavElements($params) {
		if (!isset($params['project']) || !is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}
		$result = new PluginResult($this);
		$arr = array(
			'title'=>'Frs',
			'elements'=> array(
			array('link'=>'/gf/project/'.$params['project']->getUnixName().'/frs/admin/',
				'description'=>Language::getSessText('Frs.Admin'))
			)
		);
		$result->setResult($arr);
		return $result;
	}

	private function getProjectReportingNavElements($params) {
		if (!isset($params['project']) || !is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}
		$result = new PluginResult($this);
		$arr = array(
			'title'=>Language::getSessText('Frs.TabString'),
			'elements'=> array(
			array('link'=>'/gf/project/'.$params['project']->getUnixName().'/frs/?action=FrsReport',
				'description'=> Language::getSessText('FrsReporting.Title'))
			)
		);
		$result->setResult($arr);
		return $result;
	}

	/**
	 * returns the part of the html text that goes into the project public area
	 *
	 * @param array $params needed params to build the html
	 * @return string	the builded html result
	 */	
	private function _buildProjectSummary($params) {
		if (!isset($params['project']) || !is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}
		$result = new PluginResult($this);

	$value = $params['theme']->infoTableTop(Language::getSessText('Project.LatestFRS'));
	$unix_name = $params['project']->getUnixName();
	$project_id =  $params['project']->getPrimaryKey();
	$project_name = $params['project']->getUnixName();
	$theme = $params['theme'];

	$value .= '
	<table cellspacing="1" cellpadding="5" width="100%" border="0">
		<tr>
		<td align="left">
			'.Language::getSessText('Frs.Package').'
		</td>
		<td align="center">
			'.Language::getSessText('Frs.Version').'
		</td>
		<td align="center">
			'.Language::getSessText('Frs.Date').'
		</td>
		<td align="center">
			'.Language::getSessText('Frs.NotesMonitor').'
		</td>
		<td align="center">
			'.Language::getSessText('Frs.Download').'
		</td>
		</tr>';

		//
		//  Members of projects can see all packages
		//  Non-members can only see public packages
		//
		$user = Session::getSession()->getUser();
		if (Session::getSession()->isLoggedIn()) {
			if (($user->isProjectMember($params['project'])) || (Session::getSession()->isSiteAdmin())) {
				$public='';
			} else {
				$public=' AND frs_package.is_public=true ';
			}
		} else {
			$public=' AND frs_package.is_public=true ';
		}

		// TODO -> the status id for the release should be a constant defined somewhere   -  frs_release.status_id=1
		$sql="SELECT frs_package.frs_package_id,frs_package.package_name ,frs_release.release_name,frs_release.frs_release_id,frs_release.release_date 
			FROM frs_package,frs_release WHERE frs_package.frs_package_id=frs_release.frs_package_id 
			AND frs_package.project_id='$project_id' 
			AND frs_release.status_id=1 $public ORDER BY frs_package.frs_package_id,frs_release.release_date DESC";
		
		$stmt = $params['con']->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);
		if (count($rs)<1) {
			// No releases
			$value .= '<tr><td colspan="5"><strong>'.Language::getSessText('Frs.NoReleases').'</strong></td></tr>';
		} else {
			/*
				This query actually contains ALL releases of all packages
				We will test each row and make sure the package has changed before printing the row
			*/
			$last_id = 0;
			while($rs->next()) {
				if ($rs->getInt('frs_package_id')==$last_id) {
					//same package as last iteration - don't show this release
					$last_id = $rs->getInt('frs_package_id');
				} else {
					$release_date = $rs->getString('release_date');
					$value .= '
					<tr align="center">
					<td align="left">
					<strong>' . $rs->getString('package_name') . '</strong></td>';
					// Releases to display
					$value .= '<td>'. $rs->getString('release_name') .'
					</td>
					<td>' . $release_date . '</td>
					<td><a href="'. GFUrl::getUrl()->projectURL($project_name,'shownotes','frs','',array('release_id' => $rs->getString('frs_release_id'))) . '">';
					$value .= $params['theme']->image('ic/manual16c.png',15,15,array('alt'=>Language::getSessText('Project.ReleaseNotes')));
					$value .= '</a> - <a href="'. GFUrl::getUrl()->projectURL($project_name,'monitor','frs','', array('filemodule_id'=> $rs->getString('frs_package_id'),'start' => 1)) .'">';
					$value .= $params['theme']->image('ic/mail16d.png','15','15',array('alt'=>Language::getSessText('Project.MonitorPackage')));
					$value .= '</a>
					</td>
					<td><a href="'. GFUrl::getUrl()->projectURL($project_name,'','frs','',array('release_id' => $rs->getString('frs_release_id'))) . '">'.Language::getSessText('Frs.Download').'</a></td></tr>';
					$last_id = $rs->getInt('frs_package_id');
				}
			}
		}
	$value .= '</table>
	<div align="center">
	<a href="'. GFUrl::getUrl()->projectURL($project_name,'','frs').'">['. Language::getSessText('Project.ViewProjectFiles').']</a>
	</div>'. $theme->infoTableBottom();

	$result->setResult($value);
	return $result;
	}

	function enableInProject(Project $project) {
		return;
	}

	function disableFromProject() {
		// unregister the actions
		ActionManager::unregisterListeners($this);
	}

	/**
	 * returns the possible role settings available for this plugin
	 *
	 * @return array	the roles
	 */
	function getAvailableRoleSettings() {
		$return = array();
		$return[0][] = Language::getSessText('FrsPackage.NoAccess'); //descriptions
		$return[1][] = FrsPackage::PERMLEVEL_NOACCESS; // values
		$return[0][] = Language::getSessText('FrsPackage.Read');
		$return[1][] = FrsPackage::PERMLEVEL_READ;
		$return[0][] = Language::getSessText('FrsPackage.Admin');
		$return[1][] = FrsPackage::PERMLEVEL_ADMIN;

		return $return;
	}

	/**
	 * returns the possible site-wide admin role settings for this plugin
	 *
	 * @return array	the roles
	 */
	function getAdminRoleSettings() {
		$return = array();
		$return[0][] = Language::getSessText('Frs.ProjectWideNonAdmin'); //descriptions
		$return[1][] = FrsPlugin::READ ; // values
		$return[0][] = Language::getSessText('Frs.ProjectWideAdmin');
		$return[1][] = FrsPlugin::WRITE;
		return $return;
	}

	/**
	 * 	The getRoleChoices listener - this hook will fire
	 *	when a new role is added or one is going to be updated.
	 * 	It returns an array of role settings
	 *	@return	array
	 */
	function getRoleChoices($params) {
		if (!is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}

		$roles = array();
		$roles[0]['ref_id'] = 0;
		$roles[0]['section'] = Language::getSessText('Frs.Frs');
		$roles[0]['description'] = Language::getSessText('Frs.Roles.Description');
		$roles[0]['subsection'] = '-';
		$roles[0]['settings'][0] = 'frs';
		$roles[0]['settings'][] = $this->getAdminRoleSettings();
/* COMMUNITY EXPRESS
		$c = new Criteria();
		$c->add(FrsPackagePeer::PROJECT_ID,$params['project']->getPrimaryKey());
		$c->addAscendingOrderByColumn(FrsPackagePeer::FRS_PACKAGE_ID);
		$packages = FrsPackagePeer::doSelect($c);
		foreach ($packages as $package) {
			$key = $package->getPrimaryKey();
			$roles[$key]['ref_id'] = $key;
			$roles[$key]['section'] = Language::getSessText('FrsPackage.Roles.Section'); // the displayed text in "section" field
			$roles[$key]['description'] = Language::getSessText('FrsPackage.FrsPackageWideAdmin.Description');
			$roles[$key]['subsection'] = $package->getPackageName(); // the displayed text in the "subsection" field
			$roles[$key]['settings'][0] = FrsPackage::SECTION; //$this->getName();// the section for the DB, one for all the settings
			$roles[$key]['settings'][] = $this->getAvailableRoleSettings();
		}
*/
		$result = new PluginResult($this);
		$result->setResult($roles);
		return $result;
	}

	private function updateRoleChoices($params) {
		if (!is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}
		
		$sections = array(self::SECTION,FrsPackage::SECTION);

		$project = $params['project'];
		$data = $params['data'];
		$role = $params['role'];

		foreach ($sections as $section) {
			if (!isset($data[$section])) {
				return;
			}

			foreach ($data[$section] as $ref_id => $value) {
				$c = new Criteria();
				$c->add(RoleSettingPeer::ROLE_ID, $role->getPrimaryKey());
				$c->add(RoleSettingPeer::SECTION, $section);
				$c->add(RoleSettingPeer::REF_ID, $ref_id);
				$rolesetting = RoleSettingPeer::doSelectOne($c);
				if (!is_object($rolesetting)) {
					// Enter here if we have a role setting which we didn't have previously (e.g.: activate an inactive plugin)
					$rolesetting = new RoleSetting();
					$rolesetting->setSection($section);
					$rolesetting->setRefId($ref_id);
					$rolesetting->setRoleId($role->getPrimaryKey());
				}

				// Being a project admin overrides any other value
				if (isset($data['projectadmin']) && $data['projectadmin'][0] >= Project::PROJECT_ROLE_ADMIN) {
					$value = self::WRITE;
				}			

				$rolesetting->setValue($value);
				$rolesetting->save();						
			}
		}
		return;
	}

	/**
	 * returns the settings for package status
	 *
	 * @return array
	 */
	function getStatusObserverSettings() {
		$return = array();
		$return[0][] = Language::getSessText('Frs.Private'); //descriptions
		$return[1][] = FrsPlugin::ISPRIVATE; // values
		$return[0][] = Language::getSessText('Frs.Public');
		$return[1][] = FrsPlugin::ISPUBLIC;
		return $return;		
	}		
	
	/**
	 * 	The getObserverRoleChoices listener - this hook will fire
	 *	when the observer role is modified.
	 * 	It returns an array of role settings or null
	 *	@return	array or null
	 */
	function getObserverRoleChoices($params) {	
		if (!is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}
		
		$choices = array();
		$c = new Criteria();
		$c->add(FrsPackagePeer::PROJECT_ID,$params['project']->getPrimaryKey());
		$c->addAscendingOrderByColumn(FrsPackagePeer::FRS_PACKAGE_ID);
		//retrieve the packages for the project
		$packages = FrsPackagePeer::doSelect($c);
		$i = 0;
		foreach ($packages as $package) {			
			$choices[$i]['ref_id'] = $package->getPrimaryKey();
			$choices[$i]['section'] = Language::getSessText('Frs.Privileges');
			$choices[$i]['subsection'] = $package->getPackageName();
			$choices[$i]['description'] = Language::getSessText('Frs.PackageRoles.Description');
			$choices[$i]['settings'][0] = 'frspackage';
			$choices[$i]['settings'][] = $this->getStatusObserverSettings();
			$i++;
		}

		$result = new PluginResult($this);
		$result->setResult($choices);
		return $result;
	}

	/**
	 * Retrieves the concerning data and updates its observer role settings
	 *
	 * @param array	$data
	 */
	function updateObserverSettings($params) {
		$project = $params['project'];
		$data = $params['data'];
		
		$onedata = current($data);
		while ($onedata != null) {
			$section = key($data);
			switch ($section) {
				case 'frspackage' : {
					$oneitem = current($onedata);
					while ($oneitem != null) {
						$ref_id = key($onedata);
						$value = $oneitem;
						//retrieve the package and update it
						$c = new Criteria();
						$c->add(FrsPackagePeer::FRS_PACKAGE_ID,$ref_id);
						try {
							$package = FrsPackagePeer::doSelectOne($c);
						} catch (Exception $e) {
							// something wrong happened
							echo $e->getMessage();
							next($onedata);
							$oneitem = current($onedata);
							continue;	
						}
						$package->setIsPublic($value);
						$package->save();
						next($onedata);
						$oneitem = current($onedata);					
					}		
					break;
				}
				default : { 
					//skip it
					continue;
				}
			}
			next($data);
			$onedata = current($data);
		}
	}
	
	/**
	 * Retrieve the corresponding setting (value) for the ref_id passed as argument, using the appropriate section
	 *
	 * @param int $ref_id
	 * @param string $section
	 * @return setting
	 */
	function getObserverSetting($ref_id,$section) {
		switch ($section) {
			case 'frspackage' : {
				$c = new Criteria();
				$c->add(FrsPackagePeer::FRS_PACKAGE_ID,$ref_id);
				try {
					$package = FrsPackagePeer::doSelectOne($c);
				} catch (Exception $e) {
					echo $e->getMessage();
					return '';
				}
				return $package->getIsPublic();
				break;
			}
			default : {
				return '';
			}
		}
	}
		
	/**
	 * actions to perform on project approval
	 *
	 * @param $params array
	 * @return PluginResult
	 */
	private function approveActions($params) {
		$template_project = $params['template_project'];
		$project = $params['project'];
		$con = Propel::getConnection(UserPeer::DATABASE_NAME);
		if ( (is_object($project)) && (is_object($template_project))) {
			// get the packages for the template project
			$c = new Criteria();
			$c->add(FrsPackagePeer::PROJECT_ID,$template_project->getProjectId());
			$template_packages = FrsPackagePeer::doSelect($c);
			foreach ($template_packages as $template_package) {
				// create new ones for the new project
				$new_package = $template_package->copy();
				$new_package->setProjectId($project->getProjectId());
				$new_package->save();	
				// search the role_settings and let's see if for this item we have one set

				foreach ($params['role_settings'] as $role_setting) {
					if (strstr($role_setting['section'],$this->getName()) && $role_setting['ref_id']==$template_package->getPrimaryKey()) {
						$role_id = $role_setting['role_id'];
						$section = $role_setting['section'];
						$ref_id = $new_package->getPrimaryKey();
						$value = $role_setting['value'];
						$stmt = $con->createStatement();

						// Note: when $new_package->save() is called, it creates default entries on the role_setting
						// table. We must override them.
						$sql = "DELETE FROM role_setting WHERE role_id='$role_id' AND section='$section' AND ref_id='$ref_id'";
						$rs = $stmt->executeQuery($sql);

						$sql = "insert into role_setting (role_id,section,ref_id,value) values ('$role_id','$section','$ref_id','$value')";
						$rs = $stmt->executeQuery($sql);						
					}
				}
			}
			
			// Copy the project settings
			$con->createStatement()->executeQuery('DELETE FROM frs_project_setting WHERE project_id='.$project->getProjectId());
			$con->createStatement()->executeQuery('INSERT INTO frs_project_setting(project_id, show_associations_tab, show_tags_tab) 
								SELECT '.$project->getProjectId().', show_associations_tab, show_tags_tab
								FROM frs_project_setting WHERE project_id='.$template_project->getProjectId()
							);			
		}
	}	
	
	/**
	 * Check if the user can monitor certain item
	 */	
	private function monitorCheckPermissions($params) {
		$lang = Session::getSession()->getLanguage();

		$section = $params['section'];
		$ref_id = $params['ref_id'];
		
		if ($section != 'frspackage') return;		// only deal with package monitor
		$pr = new PluginResult($this);
		
		$package = FrsPackagePeer::retrieveByPk($ref_id);
		if (is_null($package) || !$package->userCanRead()) {
			$pr->setResult(false);
		} else {
			$pr->setResult(true);
		}
		
		return $pr;
	}

	private function getProjectAuditTrail($params) {
		$project = $params['project'];
		$audit_rs = $params['audit_trail'];		// This is a recordset
		
		$section = $audit_rs->getString('section');
		$field_name = $audit_rs->getString('field_name');
		$new_value = $audit_rs->getString('new_value');
		
		if ($section != Project::SECTION) {
			// Not the project-level audit trail
			return;
		}
		
		switch ($field_name) {
		case FrsPackage::SECTION:
			if ($new_value == '_deleted_') {
				$value = $audit_rs->getString('old_value');
				$desc = Language::getSessText('FrsPackage.ProjectAudit.Deleted', array($value));
			} else {
				return;
			}
			break;
		case FrsRelease::SECTION:
			if ($new_value == '_deleted_') {
				$value = $audit_rs->getString('old_value');
				$desc = Language::getSessText('FrsRelease.ProjectAudit.Deleted', array($value));
			} else {
				return;
			}
			break;
		case 'frsfile':
			if ($new_value == '_deleted_') {
				$value = $audit_rs->getString('old_value');
				$desc = Language::getSessText('FrsFile.ProjectAudit.Deleted', array($value));
			} else {
				return;
			}
			break;
		default:
			return;
		}

		$result = new PluginResult($this);
		$result->setResult($desc);
		return $result;
	}
	
	/**
	 * Make the association (if necessary) between the SCM commit and the FRS release
	 */
	private function scmCommit($params) {
		$project = $params['project'];
		if (!$project->usesPlugin($this->getName())) return null;

		$scmcommit = $params['commit'];
		$log = $scmcommit->getMessageLog();
		
		// $params['associations'] has all the associations
		
		foreach ($params['associations'] as $association) {
			list($section, $ref_id) = $association;
			if ($section != FrsRelease::SECTION) {
				continue;
			}
			
			$frs_release_id = $ref_id;

			// see if the item belongs to the project
			$release = FrsReleasePeer::retrieveByPk($frs_release_id);
			if (is_null($release)) continue;
			
			if ($release->getFrsPackage()->getProjectId() != $project->getProjectId()) continue;
			
			// Release belongs to the project. Add entry to the association table.
			try {
				$association = new Association();
				$association->setSection(ScmCommit::SECTION);
				$association->setRefId($scmcommit->getPrimaryKey());
				$association->setToSection(FrsRelease::SECTION);
				$association->setToRefId($frs_release_id);
				$association->setComment($scmcommit->getMessageLog());
				$association->save();
			} catch (Exception $ex) {
				// This means the association is duplicated. Ignore the exception.
			}
		}
		
		return null;
	}

	function getProjectSettingsForm($params) {
		$project = $params['project'];
		if (!$project->usesPlugin($this->getName())) {
			return;
		}
		
		$setting = FrsProjectSettingPeer::retrieveByPk($project->getProjectId());
		if (!$setting) {
			$show_associations_tab = true;
			$show_tags_tab = true;
		} else {
			$show_associations_tab = $setting->getShowAssociationsTab();
			$show_tags_tab = $setting->getShowTagsTab();
		}
		
		$form = array();
		$form['title'] = Language::getSessText('Frs.ProjectSettingsTitle');
		$form['contents'] = '
			<input type="checkbox" disabled="disabled" name="frs_show_associations_tab" '.($show_associations_tab ? 'checked="checked"' : '').'/> '.Language::getSessText('Tabs.ShowAssociationsTab').' <br />
			<input type="checkbox" name="frs_show_tags_tab" '.($show_tags_tab ? 'checked="checked"' : '').'/> '.Language::getSessText('Tabs.ShowTagsTab').' <br />
		';
		
		$result = new PluginResult($this);
		$result->setResult($form);
		return $result;
	}
	
	function saveProjectSettings($params) {
		$project = $params['project'];
		$setting = FrsProjectSettingPeer::retrieveByPk($project->getProjectId());
		if (!$setting) {
			$setting = new FrsProjectSetting();
			$setting->setProject($project);
		}
		
		$setting->setShowAssociationsTab( (isset($_POST['frs_show_associations_tab']) ? true : false) );
		$setting->setShowTagsTab( (isset($_POST['frs_show_tags_tab']) ? true : false) );
		$setting->save();
	}
}

?>
