<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: aclcheck.php 816 2006-04-18 17:36:20Z marcelo $
 * @author Marcelo Mottalli	marcelo@gforgegroup.com
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
 
require_once('initialize.php');
require_once('GFConf.php');
require_once('SystemUtils.php');
require_once('core/Project.php');
require_once('plugins/scm/lib/ScmCommit.php');
require_once('plugins/scm/lib/ScmPreference.php');
require_once('plugins/scm/lib/ScmPlugin.php');
require_once('plugins/scmgit/lib/GITRepository.php');
require_once('plugins/scm/lib/ScmCommitWrapper.php');

// Get the project from the current working directory
$cwd = getcwd();
$path = explode('/', $cwd);
$project_name = $path[count($path)-1];
$project = ProjectPeer::retrieveByUnixName($project_name);
if (!$project) {
	echo 'Invalid project name: '.$project_name."\n";
}

$repository = new GITRepository($project);

$user_arr = posix_getpwuid(posix_geteuid());
$user_name = $user_arr['name'];
if (!$user_name) {
	echo 'Couldn\'t retrieve user name from the environment';
	exit(1);
}

if ($user_name == 'root') {
	exit(0);
}

// Start the session as the user that's committing
$user = UserPeer::retrieveByUnixName($user_name);
if (is_null($user)) {
	echo 'Invalid user name: '.$user_name."\n";
	exit(1);
}
Session::getSession()->setUser($user);


$input = file_get_contents ("php://stdin" );
$lines = explode("\n", $input);
$line = trim($lines[0]);

// $line should be something like
// "375afa809bf83b858db3033d51e67d1a9020633c 9c13a7a4f8c8e37747f9387ac688830b054eceed refs/heads/master"
list($old_value, $new_value, $ref_name) = explode(" ", $line);

// Get the list of commits for this push
$commits = array();
if (preg_match('/^0000000000/', $old_value)) {
	// Initial revision?
	$cmd = "git rev-list --pretty=oneline --date-order --reverse $new_value";
} else {
	$cmd = "git rev-list --pretty=oneline --date-order --reverse $new_value ^$old_value";
}

SystemUtils::execute($cmd, $commits);

// Process each commit separately
foreach ($commits as $commit_line) {
	// $commit_line is "$hash $log") {
	preg_match('/([^ ]*) (.*)/', $commit_line, $matches);
	$hash = $matches[1];
	$commit_log = $matches[2];
	
	// Get the modified files
	$modified_files = array();
	$error = SystemUtils::execute("git diff --name-only $hash $hash^ 2> /dev/null", $modified_files);
	if ($error) {
		// An error here means there's no previous revision (we're looking at the first commit) so
		// we need to get the full diff
		SystemUtils::execute("git diff --name-only $hash", $modified_files);
	}
	
	$files = array();
	foreach ($modified_files as $onefile) {
		$files[] = array(
				'path' => '/'.$onefile,
				'prev_version' => $old_value,
				'new_version' => $hash
			);
	}
	
	// Store the commit information
	$wrapper = new ScmCommitWrapper($repository, $user, $commit_log, $files);
	$wrapper->generateCommitData();
		
	// Keep track of the commit hash
	$old_value = $hash;
}


// Now, fire up the mail notifications (if configured)
$preferences = $repository->getScmPreferences();
if (!is_null($preferences) && $preferences->getMailCommits() && $project->usesPlugin('mailman')) {
	$emailScript = SystemUtils::buildFilePath(GFConf::get('system.pluginsPath'), '/scmgit/bin/post-receive-email');
	// Pipe the input to the email script
	$output = array();
	$input = trim($input);
	SystemUtils::execute("echo $input | $emailScript", $output);
}

exit(0);
?>

