<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2006-02-27
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

/**
 * 
 *	Tracker Gateway
 *
 * Leonel Corso <leonelcorso@yahoo.com.ar>
 *
 * @version   $Id
 */

require_once('propel/Propel.php');
Propel::init('../../../conf/gforge5-conf.php');
require_once('core/ProjectPeer.php');
require_once('MailParser.php');
require_once('plugins/tracker/lib/TrackerItemPeer.php');
require_once('plugins/tracker/lib/TrackerItemMessagePeer.php');


class TrackerGateway {
	/*
	 * variables
	 */
	var $From = "";
	var $FromName = "";
	var $FromEmail = "";
	var $Subject = "";
	var $ListId = "";
	var $Reference = "";
	var $MsgId = "";
	var $Sender="";
	var $Message="";
	var $IsFollowUp=0;
	var $TrackerItemId=-1;
	var $TrackerItem=null;

	function TrackerGateway() {
			
		/* Copy mail message to tmp file */
		$tmpfile = $this->copyMailTmp();
		//DBG("Tmpname: ". $tmpfile);

		/* parse email */
		$ret = $this->parseMail($tmpfile);
	
		/* Delete temp file */
		unlink($tmpfile);
	
		/* Check the return variable from parseMail */
		if (!$ret) {
			return false;
		}

		/* add the info to tracker */
		$ret = $this->addMessage();
		if (!$ret) {
			return false;
		}
	
		return true;
	}
	
	
	/**
	 * function - Copy mail(from stdin to tmp and return the tmp file
	 *
	 * @return tmp file name
	 */
	function copyMailTmp() {
		// Unfortunatly we need a temp file
		// mailparse needs to read content several times
		$tmpfile = tempnam ("/tmp", "artifact_gateway.".rand()."-".rand());
		$in = fopen("php://stdin", "r");
		$out = fopen($tmpfile, "w");
	
		while($buffer = fgets($in, 4096)) {
			fputs($out, $buffer);
		}
	
		fclose($in);
		fclose($out);
	
		return $tmpfile;
	}


	/*
	 * function - Parse mail and fill all kinds of head and body info
	 *
	 * @param  string tmp file name
	 * @return boolean true if success
	 */
	function parseMail($input_file) {
		global $argv;
		
		$mp = new MailParser($input_file);
		if (!$mp) {
			throw new Exception('Error In MailParser'); 			
		} 

		$this->FromEmail = $mp->getFromEmail();
		//
		//subjects are in this required format: '[group - tracker_name][123456] My Subject'
		//where 123456 is the tracker_item_id of the artifact message.
		//we parse that ID to get the tracker_item that this should post to
		//
		$subj = $mp->getSubject();
		if (ereg('(\[)([0-9]*)(\])',$subj,$arr)) {
		        $this->TrackerItemId=$arr[2];
			$Trackerid_end=(strpos($subj,'['.$arr[2].']')) + strlen('['.$arr[2].']');
			$this->Subject = addslashes(substr($subj,$Trackerid_end));
		} else {
			$this->Subject = addslashes($subj);
			$this->TrackerItemId=0; // Not supported at the moment
			throw new Exception("TrackerItemId needed at the moment. TrackerItem creation not supported"); 								
		}

		$body = addslashes($mp->getBody());
		// find first occurrence of the marker in the message
		$begin = strpos($body, TrackerMessagePeer::ARTIFACT_MAIL_MARKER);
		if ($begin === false) {
			throw new Exception("Response message wasn't found in your mail. Please verify that ".
							"you entered your message between the correct text markers.".
							"\nYour message was:".
							"\n".$mp->getBody());			
		}
		// get the part of the message located after the marker
		$body = substr($body, $begin+strlen(TrackerMessagePeer::ARTIFACT_MAIL_MARKER));
		// now look for the ending marker
		$end = strpos($body, TrackerMessagePeer::ARTIFACT_MAIL_MARKER);
		if ($end === false) {
			throw new Exception("Response message wasn't found in your mail. Please verify that ".
							"you entered your message between the correct text markers.".
							"\nYour message was:".
							"\n".$mp->getBody());			
		}
		$message = substr($body, 0, $end);
		$message = trim($message);
		
		// maybe the last line was "> (TrackerMessagePeer::ARTIFACT_MAIL_MARKER)". In that case, delete the last ">"
		$message = preg_replace('/>$/', '', $message);
		$this->Message = $message;
		
		return true;
	}
	
	/**
	 * Insert data into the tracker db
	 *
	 * @return - true or false
	 */
	function addMessage() {
		//
		//	get user_id
		//
		$user_id = $this->getUserId();
		/*if ($user_id) {
			//
			//	Set up this user's session before posting
			//
			session_set_new($user_id);
		}*/

		$TrackerItem =& $this->getTrackerItem();
		if (!$TrackerItem || !is_object($TrackerItem)) {			
			throw new Exception("Could Not Get TrackerItem");
		} 
		if (!$user_id) {			
			throw new Exception("Could Not Match Sender Email Address to User and TrackerItem Does Not Allow Anonymous Posts");
		}

		//
		//	Create tracker item message
		//
		
		$TrackerItemMessage = new TrackerItemMessage();
		$TrackerItemMessage->setTrackerItemId($this->TrackerItemId);
		$TrackerItemMessage->setSubmittedBy($user_id);
		$TrackerItemMessage->setBody($this->Message);			
		$TrackerItemMessage->save();
				
		return true;
	}


	/*------------------------------------------------------------------------
	 *  Utility functions 
	 *-----------------------------------------------------------------------*/

	/* Find user_id from email */
	function getUserId() {
		/*// Find User id using email
		// If no user id, user id is 0;
		$sql = "SELECT user_id FROM users 
			WHERE lower(email) ='".strtolower($this->FromEmail)."' AND status='A'";
		$res = db_query($sql);
		if (!$res || db_numrows($res) < 1) {
			return false;
		} else {
			$user_id = db_result($res,0,'user_id');
		}
		db_free_result($res);*/
	
		
		$c = new Criteria();	
		$c->add(UserPeer::EMAIL, $this->FromEmail);			
		$user=UserPeer::doSelectOne($c); 
		if (!$user) {		
			return false;
		} else {
			if($user->isActive()){
				$user_id = $user->getUserId();
			} else {
				return false;
			}
			
		}
		return $user_id;
	}

	function &getTrackerItem() {
		global $argv;

		$project =& ProjectPeer::retrieveByUnixName($argv[1]);			
		if (!$project || !is_object($project)) {
			throw new Exception("Could Not Get Project Object");
		} 			
		
		$this->TrackerItem =& TrackerItemPeer::retrieveByPk($this->TrackerItemId);
		
		return $this->TrackerItem;
	}
 
}


/**
 * Simple debugging printput
 *
 * Add this in /etc/syslog.conf and see /var/log/debug file:
 * # Debug
 * *.=debug			/var/log/debug
 * 
 */
/*function DBG($str) {
	global $debug;

	if ($debug==1) {
		system("echo \"artifact: ".$str."\n\" >> /tmp/tracker.log");
		syslog(LOG_DEBUG, "artifact_gateway: ". $str);
	} else if ($debug==2) {
		echo $str."\n";
	}
}*/
 

/* Main routine */
$debug = 0;
$myTrackerGateway = new TrackerGateway();
/*if ($myTrackerGateway->isError()) {
	if ($myTrackerGateway->FromEmail) {
		mail ($myTrackerGateway->FromEmail,'Tracker Post Rejected',$myTrackerGateway->getErrorMessage());
	}
	//DBG('Final Message: '.$myTrackerGateway->getErrorMessage());
} else {
	DBG("Success!!");
}*/

?>