<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2005-11-11
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once 'plugins/tracker/lib/om/BaseTracker.php';
require_once("plugins/tracker/lib/TrackerExtraField.php");
require_once("exceptions/PermissionDeniedException.php");
require_once("GFPermission.interface.php");
require_once("plugin/ActionManager.php");
require_once("SystemUtils.php");
require_once('lib/core/Monitor.php');
require_once("plugins/tracker/lib/TrackerEfeTransitionTrigger.php");



/** 
 * The skeleton for this class was autogenerated by Propel  on:
 *
 * [Wed Nov 16 20:42:33 2005]
 *
 * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package plugins.tracker.lib 
 */
class Tracker extends BaseTracker implements GFPermission {

	const STATUS_OPEN = 1;
	const STATUS_CLOSED = 0;

	const DATATYPE_TRACKER = 1;
	const DATATYPE_TASK = 2;
	const DATATYPE_KB = 3;

	const PERMLEVEL_NONE = -1;
	const PERMLEVEL_READ = 0; //can view all, unless restrict browse
	const PERMLEVEL_TECH = 1; //can view/edit all
	const PERMLEVEL_TECH_ADMIN = 2; //can read/edit all + be assignee
	const PERMLEVEL_ADMIN = 3; //can read/edit all + NOT assignee

	const SECTION = 'tracker';

	const PRIORITY_VERYHIGH = 1;
	const PRIORITY_HIGH = 2;
	const PRIORITY_MEDIUM = 3;
	const PRIORITY_LOW = 4;
	const PRIORITY_VERYLOW = 5;

	private $technicians;

	/**
	 * isPublic - true/false
	 * @return boolean
	 */
	function isPublic() {
		return $this->getIsPublic() && $this->getProject()->getIsPublic();
	}

	/**
	 * Specifies if the current user can create an entry in the database for this object
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanCreate() {
		if ($this->userIsAdmin()) {
			return true;
		}

		return false;
	}

	/**
	 * If this person is an admin for this object
	 * @return boolean
	 */
	public function userIsAdmin() {
		$perm = $this->getProject()->getPermission();
		if ( ($perm->check(Tracker::SECTION, 0, Tracker::PERMLEVEL_ADMIN))) { // normal permission
			return true;
		}
		if ( ($perm->check(Tracker::SECTION, $this->getTrackerId(), Tracker::PERMLEVEL_TECH_ADMIN))) { // normal permission
			return true;
		}
		return false;
	}

	/**
	 * Specifies if the current user can update this object's entry in the database
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanUpdate() {
		return $this->userIsAdmin();
	}
	
	/**
	 * Specifies if the current user can delete this object's entry from the database
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanDelete() {
		return $this->userIsAdmin();
	}

	/**
	 * Specifies if the current user has permissions to access this object
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanRead() {
		if ($this->isPublic()) {
			return true;
		}		
		if ($this->userIsAdmin()) {
			return true;
		}
		$perm = $this->getProject()->getPermission();
		if ( ($perm->check(Tracker::SECTION, $this->getTrackerId() , Tracker::PERMLEVEL_READ)) ) { // read permission
			return true;
		}		
		return false;
	}


	
	/**
	 * Removes this object from datastore and sets delete attribute. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 * 
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 * @see Tracker::setDeleted()
	 * @see Tracker::isDeleted()
	 */
	function delete($con = null) {
		if (!$this->userCanDelete()) {
			throw new PermissionDeniedException("Could not delete Tracker object: permission denied");
		}
		//delete monitors
		$crit=new Criteria();
		$crit->add(MonitorPeer::SECTION,self::SECTION);
		$crit->add(MonitorPeer::REF_ID,$this->getTrackerId());
		MonitorPeer::doDelete($crit);
		
		// Create the project-level audit trail (see comments on TrackerItem class)
		$audit = new AuditTrail();
		$audit->setSection(Project::SECTION);
		$audit->setRefId($this->getProject()->getProjectId());
		$audit->setFieldName(self::SECTION);
		$audit->setNewValue('_deleted_');		// mark the tracker item as deleted
		$audit->setOldValue($this->getTrackerName());
		$audit->setUser(Session::getSession()->getUser());
		$audit->save();


		ActionManager::notify("TRACKER_DELETE", $this);
		parent::delete($con);		
	}

	/**
	 * Stores the object in the database using the Propel methods. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 *
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 */
	function save($con = null) {
		if ($this->isNew() && !$this->userCanCreate()) {
			throw new PermissionDeniedException("Could not create Tracker object: permission denied");
		} else if (!$this->userCanUpdate()) {
			throw new PermissionDeniedException("Could not update Tracker object: permission denied");
		}
		$errors=array();
		if (strlen($this->tracker_name) < 4) {
			$errors['tracker_name'] = Language::getSessText('Tracker.EmptyTrackerName');
		}
		if (strlen($this->description) < 4) {
			$errors['description'] = Language::getSessText('Tracker.EmptyDescription');
		}
		if ($this->email_all_updates){
			$invalid_emails = Validator::validateEmailList($this->email_address);
			if (count($invalid_emails) > 0 || empty($this->email_address) ){
				$invalid_str = implode(', ', $invalid_emails);
				$errors['email_address'] = Language::getSessText('Tracker.InvalidEmail', array($invalid_str));
			}
		} else {
			if(!empty($this->email_address)){
				$invalid_emails = Validator::validateEmailList($this->email_address);
				if(count($invalid_emails) > 0){
					$invalid_str = implode(', ', $invalid_emails);
					$errors['email_address'] = Language::getSessText('Tracker.InvalidEmail', array($invalid_str));
				}
			}
		}
		if (count($errors) > 0) {
			throw new InvalidDataException($errors);
		}
		$createrolesettings = $this->isNew();

		parent::save($con);

		if ($createrolesettings) {
			// Default Permission Level should be PERMLEVEL_TECH so that this
			// works with the MS Plugin
			$con = Propel::getConnection(TrackerPeer::DATABASE_NAME);
			$sql="INSERT INTO role_setting (role_id,section,ref_id,value) 
				SELECT role_id,'".self::SECTION."','".$this->getPrimaryKey()."',
				'".self::PERMLEVEL_TECH."' 
				FROM role WHERE project_id='".$this->getProject()->getProjectId()."'";
			$stmt = $con->createStatement();
			$rs = $stmt->executeQuery($sql);
		}

		ActionManager::notify("TRACKER_SAVE", $this);
	}
	
	function cloneFields(Tracker $tracker_from) {
		$efs = $tracker_from->getTrackerExtraFields();
		
		$extra_field_map = array();
		$extra_field_element_map = array();
		
		// Get a mapping between the projects' roles
		$role_mapping = array();
		$from_roles = $tracker_from->getProject()->getRoles();
		$to_roles = $this->getProject()->getRoles();
		
		foreach ($from_roles as $from_role) {
			// Find the matching role in this project (if any)
			foreach ($to_roles as $to_role) {
				if ($from_role->getRoleName() == $to_role->getRoleName()) {
					if(!in_array($to_role->getRoleId(), $role_mapping)){
						$role_mapping[$from_role->getRoleId()] = $to_role->getRoleId();
					}
				}
			}
		}

		// clone the extra fields
		foreach ($efs as $ef) {
			//$cloned_ef = $ef->copy();
			// If we call $ef->copy(), the alias may get screwed since it tries to assign the alias
			// of the original item, and it may have random numbers on it
			$cloned_ef = new TrackerExtraField();
			$cloned_ef->setTracker($this);
			$cloned_ef->setFieldName($ef->getFieldName());	// Note that this also sets the alias
			$cloned_ef->setAlias($ef->getAlias());	// Note that we override the alias
			$cloned_ef->setFieldType($ef->getFieldType());
			$cloned_ef->setAttribute1($ef->getAttribute1());
			$cloned_ef->setAttribute2($ef->getAttribute2());
			$cloned_ef->setIsRequired($ef->getIsRequired());
			$cloned_ef->setFieldOrder($ef->getFieldOrder());
			$cloned_ef->setUseInQuickBrowse($ef->getUseInQuickBrowse());
			$cloned_ef->setShowOnSubmit($ef->getShowOnSubmit());
			// Only clone auto_assign if we're under the same project
			if ($this->getProjectId() != $tracker_from->getProjectId()) {
				$cloned_ef->setUseAutoAssign(false);
			} else {
				$cloned_ef->setUseAutoAssign($ef->getUseAutoAssign());
			}
			$cloned_ef->setUseWorkflow($ef->getUseWorkflow());
			$cloned_ef->setUseCommitRule($ef->getUseCommitRule());
			$cloned_ef->setDefaultValue($ef->getDefaultValue());
			$cloned_ef->save();
			
			$extra_field_map[$ef->getTrackerExtraFieldId()] = $cloned_ef->getTrackerExtraFieldId();
			
			// Now clone the field's elements
			$elems = $ef->getTrackerExtraFieldElements();
			foreach ($elems as $elem) {
				$cloned_elem = $elem->copy();
				$cloned_elem->setTrackerExtraField($cloned_ef);
				if ($this->getProjectId() != $tracker_from->getProjectId()) {
					$cloned_elem->setAutoAssignTo(User::UNDEFINED_USER_ID);
				}
				$cloned_elem->save();
				
				$extra_field_element_map[$elem->getElementId()] = $cloned_elem->getElementId();
			}
		}

		// Clone the workflow fields
		$c = new Criteria();
		$c->addJoin(TrackerExtraFieldElementTransitionPeer::ELEMENT_ID, TrackerExtraFieldElementPeer::ELEMENT_ID);
		$c->addJoin(TrackerExtraFieldElementPeer::TRACKER_EXTRA_FIELD_ID, TrackerExtraFieldPeer::TRACKER_EXTRA_FIELD_ID);
		$c->add(TrackerExtraFieldPeer::TRACKER_ID, $tracker_from->getTrackerId());
		$transitions = TrackerExtraFieldElementTransitionPeer::doSelect($c);
		foreach ($transitions as $transition) {
			$cloned_transition = new TrackerExtraFieldElementTransition();
			$cloned_transition->setElementId($extra_field_element_map[$transition->getElementId()]);
			$cloned_transition->setToElementId($extra_field_element_map[$transition->getToElementId()]);
			$cloned_transition->setNotificationMessage($transition->getNotificationMessage());
			$cloned_transition->save();
			
			// Roles
			$c = new Criteria();
			$c->add(TrackerExtraFieldElementTransitionRolePeer::TRACKER_EXTRA_FIELD_ELEMENT_TRANSITION_ID, $transition->getTrackerExtraFieldElementTransitionId());
			$roles = TrackerExtraFieldElementTransitionRolePeer::doSelect($c);
			foreach ($roles as $role) {
				if (!array_key_exists($role->getRoleId(), $role_mapping)) {
					continue;
				}
				
				$cloned_transition_role = new TrackerExtraFieldElementTransitionRole();
				$cloned_transition_role->setTrackerExtraFieldElementTransitionId($cloned_transition->getTrackerExtraFieldElementTransitionId());
				$cloned_transition_role->setRoleId($role_mapping[$role->getRoleId()]);
				$cloned_transition_role->save();
			}
			
			
			// Required fields
			$required_fields = $transition->getTrackerExtraFieldElementTransitionFields();
			foreach ($required_fields as $required_field) {
				$cloned_required_field = new TrackerExtraFieldElementTransitionField();
				$cloned_required_field->setTrackerExtraFieldElementTransitionId($cloned_transition->getTrackerExtraFieldElementTransitionId());
				$cloned_required_field->setRequiredTrackerExtraFieldId($extra_field_map[$required_field->getRequiredTrackerExtraFieldId()]);
				$cloned_required_field->save();
			}
			
			// Notifications
			$c = new Criteria();
			$c->add(TrackerExtraFieldElementTransitionNotifyPeer::TRACKER_EXTRA_FIELD_ELEMENT_TRANSITION_ID, $transition->getTrackerExtraFieldElementTransitionId());
			$notifications = TrackerExtraFieldElementTransitionNotifyPeer::doSelect($c);
			foreach ($notifications as $notification) {
				if (!array_key_exists($notification->getRoleId(), $role_mapping)) {
					continue;
				}
				$cloned_notification = new TrackerExtraFieldElementTransitionNotify();
				$cloned_notification->setTrackerExtraFieldElementTransitionId($cloned_transition->getTrackerExtraFieldElementTransitionId());
				$cloned_notification->setRoleId($role_mapping[$notification->getRoleId()]);
				$cloned_notification->save();
			}
			
			// Reassigns
			$c = new Criteria();
			$c->add(TrackerEfeTransitionReassignPeer::TRACKER_EXTRA_FIELD_ELEMENT_TRANSITION_ID, $transition->getTrackerExtraFieldElementTransitionId());
			$reassigns = TrackerEfeTransitionReassignPeer::doSelect($c);
			foreach ($reassigns as $reassign) {
				if (!array_key_exists($reassign->getAssignToRoleId(), $role_mapping)) {
					continue;
				}
				$cloned_reassign = new TrackerEfeTransitionReassign();
				$cloned_reassign->setTrackerExtraFieldElementTransitionId($cloned_transition->getTrackerExtraFieldElementTransitionId());
				$cloned_reassign->setAssignToRoleId($role_mapping[$reassign->getAssignToRoleId()]);
				$cloned_reassign->save();
			}
			
			// Transition triggers
			$c = new Criteria();
			$c->add(TrackerEfeTransitionTriggerPeer::TRACKER_EXTRA_FIELD_ELEMENT_TRANSITION_ID, $transition->getTrackerExtraFieldElementTransitionId());
			$triggers = TrackerEfeTransitionTriggerPeer::doSelect($c);
			foreach ($triggers as $trigger) {
				$cloned_trigger = new TrackerEfeTransitionTrigger();
				$cloned_trigger->setTrackerExtraFieldElementTransitionId($cloned_transition->getTrackerExtraFieldElementTransitionId());
				$cloned_trigger->setChangeTrackerExtraFieldId($extra_field_map[$trigger->getChangeTrackerExtraFieldId()]);
				$cloned_trigger->setToElementId($extra_field_element_map[$trigger->getToElementId()]);
				$cloned_trigger->save();
			}
			
		}

		
		// Clone the commit triggers
		$c = new Criteria();
		$c->add(TrackerCommitTriggerPeer::TRACKER_ID, $tracker_from->getTrackerId());
		$triggers = TrackerCommitTriggerPeer::doSelect($c);
		foreach ($triggers as $trigger) {
			if (!array_key_exists($trigger->getChangeTrackerExtraFieldId(), $extra_field_map) || !array_key_exists($trigger->getToElementId(), $extra_field_element_map)) {
				continue;
			}
			$cloned_trigger= new TrackerCommitTrigger();
			$cloned_trigger->setTracker($this);
			$cloned_trigger->setChangeTrackerExtraFieldId($extra_field_map[$trigger->getChangeTrackerExtraFieldId()]);
			$cloned_trigger->setToElementId($extra_field_element_map[$trigger->getToElementId()]);
			$cloned_trigger->save();
		}
		
		// Clone the commit reassign triggers
		$c = new Criteria();
		$c->add(TrackerCommitReassignTriggerPeer::TRACKER_ID, $tracker_from->getTrackerId());
		$triggers = TrackerCommitReassignTriggerPeer::doSelect($c);
		foreach ($triggers as $trigger) {
			// See if there's a matching role on the destination project
			if (!array_key_exists($trigger->getRole()->getRoleId(), $role_mapping)) {
				continue;
			}

			$cloned_trigger = new TrackerCommitReassignTrigger();
			$cloned_trigger->setTracker($this);
			$cloned_trigger->setAssignToRoleId($role_mapping[$trigger->getRole()->getRoleId()]);
			$cloned_trigger->save();
		}

	}

	function buildDefaultFields() {
		$efs = $this->getTrackerExtraFields();
		$efarr=array();
		foreach ($efs as $ef) {
			$efarr[$ef->getAlias()]=$ef;
		}

		if (!isset($efarr['duration'])) {
			$duration = new TrackerExtraField();
			$duration->setTracker($this);
			$duration->setFieldName('Duration (Days)');
			$duration->setAlias('duration');
			$duration->setFieldType(TrackerExtraField::TYPE_NUMERIC);
			$duration->setAttribute1('3');
			$duration->setAttribute2('3');
			$duration->setIsRequired(false);
			$duration->setUseInQuickBrowse(false);
			$duration->setShowOnSubmit(true);
			$duration->save();
		}
		if (!isset($efarr['percentcomplete'])) {
			$complete = new TrackerExtraField();
			$complete->setTracker($this);
			$complete->setFieldName('Percent Complete (0-100)');
			$complete->setAlias('percentcomplete');
			$complete->setFieldType(TrackerExtraField::TYPE_NUMERIC);
			$complete->setAttribute1('3');
			$complete->setAttribute2('3');
			$complete->setIsRequired(false);
			$complete->setUseInQuickBrowse(false);
			$complete->setShowOnSubmit(true);
			$complete->save();
		}
		if (!isset($efarr['estimatedeffort'])) {
			$effort = new TrackerExtraField();
			$effort->setTracker($this);
			$effort->setFieldName('Estimated Effort (Hours)');
			$effort->setAlias('estimatedeffort');
			$effort->setFieldType(TrackerExtraField::TYPE_NUMERIC);
			$effort->setAttribute1('4');
			$effort->setAttribute2('4');
			$effort->setIsRequired(false);
			$effort->setUseInQuickBrowse(false);
			$effort->setShowOnSubmit(true);
			$effort->save();
		}
		if (!isset($efarr['status'])) {
			$status = new TrackerExtraField();
			$status->setTracker($this);
			$status->setFieldName('Status');
			$status->setAlias('status');
			$status->setFieldType(TrackerExtraField::TYPE_STATUS);
		//	$status->setAttribute1('4');
		//	$status->setAttribute2('4');
			$status->setIsRequired(false);
			$status->setUseInQuickBrowse(true);
			$status->setShowOnSubmit(false);

			$efe = new TrackerExtraFieldElement();
			$status->addTrackerExtraFieldElement($efe);
			$efe->setElementName('Open');
			$efe->setStatusId(Tracker::STATUS_OPEN);
			$efe->setDefaultSelected(true);
			$efe->setSortOrder(1);

			$efe = new TrackerExtraFieldElement();
			$status->addTrackerExtraFieldElement($efe);
			$efe->setElementName('Closed');
			$efe->setStatusId(Tracker::STATUS_CLOSED);
			$efe->setDefaultSelected(false);
			$efe->setSortOrder(2);

			$status->save();
		}
	}

	/**
	 * returns arranged where each element is an array, cannot
	 * be directly used by trackeritem->addExtraFIelds() or updateExtraFields
	 * pass $arranged=true if you want it to be arranged for those functions
	 */
	function getTrackerDefaultSelected($criteria,$arranged=false) {
		$fields = $this->getTrackerExtraFields($criteria);
		$selected=array();
		foreach ($fields as $field) {

			$elements = $field->getTrackerExtraFieldElements();
			foreach ($elements as $element) {
				if ($element->getDefaultSelected()) {
					if ($arranged) {
						if ($field->getFieldType() == TrackerExtraField::TYPE_CHECKBOX ||
							$field->getFieldType() == TrackerExtraField::TYPE_MULTISELECT) {
							//need to be arrays
							$selected[$field->getTrackerExtraFieldId()][]=$element->getId();
						} else {
							$selected[$field->getTrackerExtraFieldId()]=$element->getId();
						}
					} else {
						$selected[$field->getTrackerExtraFieldId()][]=$element->getId();
					}
				}
			}
			//
			//	may not have an element selected for default
			//
			if (!isset($selected[$field->getTrackerExtraFieldId()])) {
				if ($field->getFieldType() == TrackerExtraField::TYPE_DATE
					|| $field->getFieldType() == TrackerExtraField::TYPE_TEXTAREA) {
					if ($arranged) {
						$selected[$field->getTrackerExtraFieldId()]='';
					} else {
						$selected[$field->getTrackerExtraFieldId()][]='';
					}
				} elseif ($field->getFieldType() == TrackerExtraField::TYPE_NUMERIC
					|| $field->getFieldType() == TrackerExtraField::TYPE_TEXT) {
					if ($arranged) {
						$selected[$field->getTrackerExtraFieldId()]=$field->getDefaultValue();
					} else {
						$selected[$field->getTrackerExtraFieldId()][]=$field->getDefaultValue();
					}
					
				} else {
					if ($arranged) {
						if ($field->getFieldType() == TrackerExtraField::TYPE_CHECKBOX ||
							$field->getFieldType() == TrackerExtraField::TYPE_MULTISELECT) {
							//need to be arrays
							$selected[$field->getTrackerExtraFieldId()][]=100;
						} else {
							$selected[$field->getTrackerExtraFieldId()]=100;
						}
					} else {
						$selected[$field->getTrackerExtraFieldId()][]=100;
					}
				}
			}
		}
		$selected['PRIORITY']=3;
		$selected['ASSIGNEDTO']=array(100);
		return $selected;
	}

	/**
	 *	getStatuses - return an associative array with the statuses
	 *
	 *	@return array	
	 */
	static function getStatuses() {
		$language = Session::getSession()->getLanguage(); // get the language object for this session
		//set the various statuses
		$open = $language->getText('tracker.status.open');
		$closed = $language->getText('tracker.status.closed');
		$statuses = array (Tracker::STATUS_OPEN=>$open,Tracker::STATUS_CLOSED=>$closed);
		return $statuses;
	}
	
	/**
	 *	getPriorities - return an associative array with the priorities
	 *
	 *	@return array	
	 */
	static function getPriorities() {	
		$language = Session::getSession()->getLanguage(); // get the language object for this session
		//set the various priorities
		$veryhigh = $language->getText('tracker.priority.veryhigh');
		$high = $language->getText('tracker.priority.high');
		$medium = $language->getText('tracker.priority.medium');
		$low = $language->getText('tracker.priority.low');
		$verylow = $language->getText('tracker.priority.verylow');
		$priorities = array(Tracker::PRIORITY_VERYHIGH=>$veryhigh,
		Tracker::PRIORITY_HIGH=>$high,
		Tracker::PRIORITY_MEDIUM=>$medium,
		Tracker::PRIORITY_LOW=>$low,
		Tracker::PRIORITY_VERYLOW=>$verylow); // this is an associative array with the priorities and their string representation
		return $priorities;
	}	
	
	/**
	 *	getStatusName - construct a string with the status of the task.
	 *
	 *	@param	int	StatusId
	 *	@return string	Status name
	 */
	function getStatusName($StatusId) {

		$id = $StatusId; // get the status_id

		$statuses = $this->getStatuses();
		if (array_key_exists($StatusId, $statuses)) {
			return $statuses[$StatusId];	
		} else {
			return "--"; // this really shouldn't happen
		}
	}	

	function getTechnicians() {
		if (!isset($this->technicians)) {
			$sql = "SELECT * FROM \"user\" WHERE user_id IN
					(
						SELECT user_project.user_id 
						FROM user_project, user_project_role, role_setting 
						WHERE user_project.user_project_id=user_project_role.user_project_id
						AND user_project_role.role_id=role_setting.role_id
						AND role_setting.ref_id='".$this->getTrackerId()."'
						AND role_setting.section='". self::SECTION ."'
						AND value IN (1,2)
						UNION SELECT member_user_id AS user_id FROM usergroup_user WHERE usergroup_id IN
						(
							SELECT user_project.user_id AS usergroup_id
							FROM user_project, user_project_role, role_setting 
							WHERE user_project.user_project_id=user_project_role.user_project_id
							AND user_project_role.role_id=role_setting.role_id
							AND role_setting.ref_id='".$this->getTrackerId()."'
							AND role_setting.section='". self::SECTION ."'
							AND value IN (1,2)
						)
					)
					AND is_group = 'f'
					ORDER BY firstname ASC, lastname ASC;
					";

			$con = Propel::getConnection(UserPeer::DATABASE_NAME);
			$stmt = $con->createStatement();
			$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);
	
			$this->technicians=UserPeer::populateObjects($rs);
		}
		return $this->technicians;
	}

	/**
	 * Tells if the user is monitoring this tracker
	 * @return bool
	 */
	public function userIsMonitoring(User $user) {
		$c = new Criteria();
		$c->add(MonitorPeer::SECTION, self::SECTION);
		$c->add(MonitorPeer::REF_ID, $this->getPrimaryKey());
		$c->add(MonitorPeer::USER_ID, $user->getUserId());
		$mon = MonitorPeer::doSelectOne($c);
		return (!is_null($mon)) ? true : false;
	}

	public function getTrackerLink() {
		return '<a href="/gf/project/'.$this->getProject()->getUnixName().'/tracker/?action=TrackerItemBrowse&amp;tracker_id='.$this->getTrackerId().'">'.$this->getTrackerName().'</a>';
	}
	
	public function isTemplateTracker() {
		return $this->getProject()->getIsTemplate();
	}
	
	/**
	 * If this tracker is in a template project, return all the trackers with the same
	 * name that reference this project as a template project
	 * @return array
	 */
	public function getDependingTrackers() {
		$this_project = $this->getProject();
		
		if (!$this_project->getIsTemplate()) {
			return array();
		}
		
		$c = new Criteria();
		$c->add(ProjectPeer::TEMPLATE_PROJECT_ID, $this_project->getProjectId());
		$c->addJoin(TrackerPeer::PROJECT_ID, ProjectPeer::PROJECT_ID);
		$c->add(TrackerPeer::TRACKER_NAME, $this->getTrackerName());
		return TrackerPeer::doSelect($c);
	}

}

?>
