<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('plugins/tracker/lib/TrackerItemPeer.php');
require_once('plugins/tracker/lib/TrackerItemSavedQuery.php');
require_once('plugins/tracker/lib/TrackerItemQuickQuery.php');
require_once('plugins/tracker/lib/TrackerQuery.php');
require_once('plugins/tracker/lib/TrackerItem.php');
require_once('exceptions/PermissionDeniedException.php');

$session = Session::getSession();
$Lang = $session->getLanguage();

$Project = GFUrl::getUrl()->getProject();
$Tracker = TrackerPeer::retrieveByPk($_REQUEST['tracker_id']);
if (!$Tracker || !is_object($Tracker) || $Tracker->getProject()->getProjectId() != $Project->getProjectId() || !$Tracker->userCanRead()) {
        throw new PermissionDeniedException();
}

if (isset($_GET['tracker_query_id'])) {
	if (!$session->isLoggedIn()) {
		throw new PermissionDeniedException();
	}
	$tsq = TrackerQueryPeer::retrieveByPk($_GET['tracker_query_id']);
	if ($tsq->getUserId() != $session->getUser()->getUserId()) {
		throw new PermissionDeniedException();
	}
	$tq = new TrackerItemSavedQuery($Tracker, $tsq);
} else {
	throw new PermissionDeniedException();
}

$totalcount = $tq->getTrackerItemCount();
$results = $tq->getTrackerItems();

$dom = new DOMDocument('1.0', 'iso-8859-1');
$domTracker = $dom->createElement('Tracker');
$domTracker->setAttribute('id', $Tracker->getTrackerId());
$domTrackerNode = $dom->appendChild($domTracker);
$domTrackerName = $dom->createElement('TrackerName', $Tracker->getTrackerName());
$domTrackerNode->appendChild($domTrackerName);

foreach ($results as $query) {
	$domTrackerItem = $dom->createElement('TrackerItem');
	$domTrackerItem->setAttribute('id', $query->getTrackerItemId());
	
	$summary = $dom->createElement('Summary', $query->getSummary());
	$domTrackerItem->appendChild($summary);

	if ($tq->showColumn('priority')) {
		$priority = $dom->createElement('Priority', $query->getPriority());
		$domTrackerItem->appendChild($priority);
	}

	if ($tq->showColumn('assigned_to')) {
		$asses = $query->getTrackerItemAssignees();
		$assignees = $dom->createElement('Assignees');
		foreach ($asses as $ass) {
			$assignee = $dom->createElement('Assignee', trim($ass->getUser()->getUnixName()));
			$assignee->setAttribute('id', $ass->getUser()->getUserId());
			$assignees->appendChild($assignee);
		}
		$domTrackerItem->appendChild($assignees);
	}	

	if ($tq->showColumn('submitted_by')) {
		$subUser = UserPeer::retrieveByPk($query->getSubmittedBy());
		$submitter = $dom->createElement('Submitter', $subUser->getUnixName());
		$submitter->setAttribute('id', $subUser->getUserId());
		$domTrackerItem->appendChild($submitter);
	}
	// Begin Extra Fields //
	$extrafields = $dom->createElement('ExtraFields');

	// Get the extra fields from the Tracker
	$efs=$Tracker->getTrackerExtraFields();
	$efcache=array();
	$efnames=array();
	foreach ($efs as $ef) {
		$efcache[$ef->getTrackerExtraFieldId()]=$ef;
		if ($ef->getUseInQuickBrowse()) {
			$efnames[$ef->getTrackerExtraFieldId()] = $ef->getFieldName();
		}
	}
	$colarray=array();
	$efdatas=$query->getTrackerExtraFieldDatas();
	foreach($efdatas as $efdata) {
		//Members type needs an special treatment
		$field_type = $efcache[$efdata->getTrackerExtraFieldId()]->getFieldType();
		if($field_type == TrackerExtraField::TYPE_MEMBERS){
			$member = UserPeer::retrieveByPk($efdata->getFieldData());
			$colarray[$efdata->getTrackerExtraFieldId()][] = $member->getUnixName();
		} else {
			$colarray[$efdata->getTrackerExtraFieldId()][]=$efcache[$efdata->getTrackerExtraFieldId()]->translateFieldDataToName($efdata->getFieldData());
		}
	}
	reset($efs);
	foreach ($efs as $ef) {
		if ($ef->getUseInQuickBrowse()) {
			if (isset($colarray[$ef->getTrackerExtraFieldId()]) && count($colarray[$ef->getTrackerExtraFieldId()]) > 1) {
				$extrafield = $dom->createElement('ExtraField');
				$extrafield->setAttribute('name', $efnames[$ef->getTrackerExtraFieldId()]);
				foreach ($colarray[$ef->getTrackerExtraFieldId()] as $f) {
					$value = $dom->createElement('Value', $f);
					$extrafield->appendChild($value);
				}
			} else {
				if (isset($colarray[$ef->getTrackerExtraFieldId()])) {
					if ($ef->getFieldType() == TrackerExtraField::TYPE_DATE) {
						$extrafield = $dom->createElement('ExtraField', date('Y-m-d', strtotime($colarray[$ef->getTrackerExtraFieldId()][0])));
						$extrafield->setAttribute('name', $efnames[$ef->getTrackerExtraFieldId()]);
					} else {
						$extrafield = $dom->createElement('ExtraField', $colarray[$ef->getTrackerExtraFieldId()][0]);
						$extrafield->setAttribute('name', $efnames[$ef->getTrackerExtraFieldId()]);						
					}
				} else {
					$extrafield = $dom->createElement('ExtraField', 'None');
					$extrafield->setAttribute('name', $efnames[$ef->getTrackerExtraFieldId()]);
				}
			}
			$extrafields->appendChild($extrafield);
		}
	}
	
	$domTrackerItem->appendChild($extrafields);
	// End Extra Fields //

	if ($tq->showColumn('open_date')) {
		$opendate = $dom->createElement('OpenDate', $query->getOpenDate('Y-m-d'));
		$domTrackerItem->appendChild($opendate);
	}
	if ($tq->showColumn('close_date')) {
		$date =$query->getCloseDate('Y-m-d');
		$closedate = $dom->createElement('CloseDate', $date);
		$domTrackerItem->appendChild($closedate);
	}
	if ($tq->showColumn('last_modified_date')) {
		$lastmodif = $dom->createElement('LastModifiedDate', $query->getLastModifiedDate('Y-m-d'));
		$domTrackerItem->appendChild($lastmodif);
	}
	$domTrackerNode->appendChild($domTrackerItem);
}
header("Content-Type: text/xml");
echo $dom->saveXML();
?>
