<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Ruben Gutierrez	ruben@gforgegroup.com
 * @date 2006-03-16
 *
 * SOAP plugin for GForge5.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
 
require_once('plugins/xmlcompatibility/lib/soap/artifact/ArtifactFileTypes.php');
require_once('plugins/tracker/lib/TrackerItemPeer.php');

$server->register(
	'getArtifactFiles',
	array(
		'session_ser'=>'xsd:string',
		'group_id'=>'xsd:int',
		'group_artifact_id'=>'xsd:int',
		'artifact_id'=>'xsd:int'
	),
	array('getArtifactFilesResponse'=>'tns:ArrayOfArtifactFile'),
	$uri,$uri.'#getArtifactFiles','rpc','encoded'
);

// This is for retrieving a single file base64-encoded
$server->register(
	'getArtifactFileData',
	array(
		'session_ser'=>'xsd:string',
		'group_id'=>'xsd:int',
		'group_artifact_id'=>'xsd:int',
		'artifact_id'=>'xsd:int',
		'file_id'=>'xsd:int'
	),
	array('getArtifactFileDataResponse'=>'xsd:string'),
	$uri,$uri.'#getArtifactFileData','rpc','encoded'
);

$server->register(
	'addArtifactFile',
	array(
		'session_ser'=>'xsd:string',
		'group_id'=>'xsd:int',
		'group_artifact_id'=>'xsd:int',
		'artifact_id'=>'xsd:int',
		'base64_contents'=>'xsd:string',
		'description'=>'xsd:string',
		'filename'=>'xsd:string',
		'filetype'=>'xsd:string'
		),
	array('addArtifactFileResponse'=>'xsd:int'),
	$uri,$uri.'#addArtifactFile','rpc','encoded'
);

$server->register(
	'artifactFileDelete',
	array(
		'session_ser'=>'xsd:string',
		'group_id'=>'xsd:int',
		'group_artifact_id'=>'xsd:int',
		'artifact_id'=>'xsd:int',
		'file_id'=>'xsd:int'
	),
	array('artifactFileDeleteResponse'=>'xsd:boolean'),
	$uri,$uri.'#artifactFileDeleteResponse','rpc','encoded'
);

//
//	getArtifactFiles
//
function getArtifactFiles($session_ser, $group_id, $group_artifact_id, $artifact_id) {
	try {
		$session = continueSession($session_ser);
	
		$TrackerItem = TrackerItemPeer::retrieveByPk($artifact_id);
		if (!$TrackerItem || !is_object($TrackerItem)) {
			return new soap_fault('', 'getArtifactFiles',
								'Could Not Get Artifact Files: No Such Artifact',
								'Could Not Get Artifact Files: No Such Artifact');
		}
		$Tracker=$TrackerItem->getTracker();
	
		if ($Tracker->getProject()->getProjectId() != $group_id
			|| $Tracker->getTrackerId() != $group_artifact_id) {
			return new soap_fault('', 'getArtifactFiles',
								'Could Not Get Artifact Files: No Such Artifact',
								'Could Not Get Artifact Files: No Such Artifact');
		}
	
		$c = new Criteria();
		$c->add(FilesystemPeer::SECTION,TrackerItem::SECTION);
		$c->add(FilesystemPeer::REF_ID, $TrackerItem->getTrackerItemId());
		$Files = FilesystemPeer::doSelect($c);
	
		return trackerItemFilesToSoap($Files);
	} catch (Exception $ex) {
		return new soap_fault ('', 'getArtifactFiles',
							'Could Not Get Artifact Files: ' . $ex->getMessage(),
							'Could Not Get Artifact Files: ' . $ex->getMessage());
	}
}

//
//	getArtifactFileData
//
function getArtifactFileData($session_ser, $group_id, $group_artifact_id, $artifact_id, $file_id) {
	try {
		$session = continueSession($session_ser);
	
		$TrackerItem = TrackerItemPeer::retrieveByPk($artifact_id);
		if (!$TrackerItem || !is_object($TrackerItem)) {
			return new soap_fault('', 'getArtifactFileData',
								'Could Not Get Artifact File Data: No Such Artifact',
								'Could Not Get Artifact File Data: No Such Artifact');
		}
		$Tracker=$TrackerItem->getTracker();
		if ($Tracker->getProject()->getProjectId() != $group_id
			|| $Tracker->getTrackerId() != $group_artifact_id) {
			return new soap_fault('', 'getArtifactFileData',
								'Could Not Get Artifact File Data: No Such Artifact',
								'Could Not Get Artifact File Data: No Such Artifact');
		}
		$fs = FilesystemPeer::retrieveByPk($file_id);
		if (!is_object($fs) 
			|| $fs->getSection() != TrackerItem::SECTION
			|| $fs->getRefId() != $TrackerItem->getTrackerItemId()) {
			return new soap_fault('', 'getArtifactFiles',
								'Could Not Get Artifact File Data: No Such File',
								'Could Not Get Artifact File Data: No Such File');
			
		}
		//send file encoded in base64
		return base64_encode($fs->getFileData());
	} catch (Exception $ex) {
		return new soap_fault ('', 'getArtifactFileData',
							'Could Not Get Artifact File Data: ' . $ex->getMessage(),
							'Could Not Get Artifact File Data: ' . $ex->getMessage());
	}
}

//
//	artifactFileDelete
//
function artifactFileDelete($session_ser, $group_id, $group_artifact_id, $artifact_id, $file_id) {
	try {
		$session = continueSession($session_ser);
	
		$TrackerItem = TrackerItemPeer::retrieveByPk($artifact_id);
		if (!$TrackerItem || !is_object($TrackerItem)) {
			return new soap_fault('', 'artifactFileDelete',
								'Could Not Delete Artifact File: No Such Artifact',
								'Could Not Delete Artifact File: No Such Artifact');
		}
		$Tracker=$TrackerItem->getTracker();
		if (!$TrackerItem->userCanUpdate() || $Tracker->getProject()->getProjectId() != $group_id
			|| $Tracker->getTrackerId() != $group_artifact_id) {
			return new soap_fault('', 'artifactFileDelete',
								'Could Not Delete Artifact File: Permission Denied',
								'Could Not Delete Artifact File: Permission Denied');
		}
	
		$fs = FilesystemPeer::retrieveByPk($file_id);
		if (!is_object($fs) 
			|| $fs->getSection() != TrackerItem::SECTION
			|| $fs->getRefId() != $TrackerItem->getTrackerItemId()) {
			return new soap_fault('', 'artifactFileDelete',
								'Could Not Delete Artifact File: No Such File',
								'Could Not Delete Artifact File: No Such File');
			
		}
	
		$fs->delete();
		$fs = null;
		return true;
	} catch (Exception $ex) {
		return new soap_fault('', 'artifactFileDelete',
							'Could Not Delete Artifact File: '.$ex->getMessage(),
							'Could Not Delete Artifact File: '.$ex->getMessage());
	}
}

//
//	addArtifactFile
//
function addArtifactFile($session_ser, $group_id, $group_artifact_id, $artifact_id,
						$base64_contents, $description, $filename, $filetype) {
	try {
		$session = continueSession($session_ser);
	
		$TrackerItem = TrackerItemPeer::retrieveByPk($artifact_id);
		if (!$TrackerItem || !is_object($TrackerItem)) {
			return new soap_fault('', 'addArtifactFile',
								'Could Not Add Artifact File: No Such Artifact',
								'Could Not Add Artifact File: No Such Artifact');
		}
		$Tracker=$TrackerItem->getTracker();
		if (!$TrackerItem->userCanCreate() || $Tracker->getProject()->getProjectId() != $group_id
			|| $Tracker->getTrackerId() != $group_artifact_id) {
			return new soap_fault('', 'addArtifactFile',
								'Could Not Add Artifact File: Permission Denied',
								'Could Not Add Artifact File: Permission Denied');
		}
	
		$binaryData = base64_decode($base64_contents);
		$size = strlen($binaryData);
	
		$fs = new Filesystem();
		$fs->setSection(TrackerItem::SECTION);
		$fs->setRefId($TrackerItem->getTrackerItemId());
		$fs->setFileName(Validator::sanitizeString($filename));
		$fs->setFileType(Validator::sanitizeString($filetype));
		$fs->setFileSize($size);
		$fs->setPostedBy($session->getUser()->getUserId());
		$fs->save();
		$fs->setFileDataFromString($binaryData);
		return $fs->getFilesystemId();
		
	} catch (Exception $ex) {
		return new soap_fault('', 'addArtifactFile',
							'Could Not Add Artifact File: '.$ex->getMessage(),
							'Could Not Add Artifact File: '.$ex->getMessage());
	}
}

//
//	convert array of artifact files to soap data structure
//
function trackerItemFilesToSoap($Files) {
	$return = array();

	foreach ($Files as $File) {
		$return[] = array(
			'id' => $File->getFilesystemId(),
			'artifact_id' => $File->getRefId(),
			'name' => $File->getFileName(),
			'description' => $File->getFileName(),
			'filesize' => $File->getFileSize(),
			'filetype' => $File->getFileType(),
			'adddate' => 0,
			'submitted_by' => $File->getPostedBy()
		);
	}
	return $return;
}

?>
