<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Ruben Gutierrez	ruben@gforgegroup.com
 * @date 2006-03-16
 *
 * SOAP plugin for GForge5.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('plugins/xmlcompatibility/lib/soap/frs/FrsTypes.php');
require_once('plugins/frs/lib/FrsPackagePeer.php');
require_once('plugins/frs/lib/FrsReleasePeer.php');

$server->register(
	'getPackages',
	array(
		'session_ser'=>'xsd:string',
		'group_id'=>'xsd:int'),
	array('getPackagesResponse'=>'tns:ArrayOfFRSPackage'),
		$uri,$uri.'#getPackages','rpc','encoded'
);

$server->register(
	'addPackage',
	array(
		'session_ser'=>'xsd:string',
		'group_id'=>'xsd:int',
		'package_name'=>'xsd:string',
		'is_public'=>'xsd:int'),
	array('addPackageResponse'=>'xsd:int'),
		$uri,$uri.'#addPackage','rpc','encoded'
);


$server->register(
	'getReleases',
	array(
		'session_ser'=>'xsd:string',
		'group_id'=>'xsd:int',
		'package_id'=>'xsd:int'),
	array('getPackagesResponse'=>'tns:ArrayOfFRSRelease'),
		$uri,$uri.'#getReleases','rpc','encoded'
);

$server->register(
	'addRelease',
	array(
		'session_ser'=>'xsd:string',
		'group_id'=>'xsd:int',
		'package_id'=>'xsd:int',
		'name'=>'xsd:string',
		'notes'=>'xsd:string',
		'changes'=>'xsd:string',
		'release_date'=>'xsd:int'),
	array('addRelease'=>'xsd:int'),
		$uri,$uri.'#addRelease','rpc','encoded'
);


$server->register(
	'getFiles',
	array(
		'session_ser'=>'xsd:string',
		'group_id'=>'xsd:int',
		'package_id'=>'xsd:int',
		'release_id'=>'xsd:int'),
	array('getFilesResponse'=>'tns:ArrayOfFRSFile'),
		$uri,$uri.'#getFiles','rpc','encoded'
);

$server->register(
	'getFile',
	array(
		'session_ser'=>'xsd:string',
		'group_id'=>'xsd:int',
		'package_id'=>'xsd:int',
		'release_id'=>'xsd:int',
		'file_id'=>'xsd:int'),
	array('getFileResponse'=>'xsd:string'),
		$uri,$uri.'#getFile','rpc','encoded'
);

$server->register(
	'addFile',
	array(
		'session_ser'=>'xsd:string',
		'group_id'=>'xsd:int',
		'package_id'=>'xsd:int',
		'release_id'=>'xsd:int',
		'name'=>'xsd:string',
		'base64_contents'=>'xsd:string',
		'type_id'=>'xsd:int',
		'processor_id'=>'xsd:int',
		'release_time'=>'xsd:int'
		),
	array('addFile'=>'xsd:string'),
		$uri,$uri.'#addFile','rpc','encoded'
);

function getPackages($session_ser, $group_id) {
	$session = continueSession($session_ser);

	$Project = ProjectPeer::retrieveByPk($group_id);
	if (!$Project || !is_object($Project)) {
		return new soap_fault ('', 'getPackages', 'Could Not Get Group', 'Could Not Get Group');
	}

	$c = new Criteria();
	$c->add(FrsPackagePeer::PROJECT_ID, $Project->getProjectId());
	$FrsPackages = FrsPackagePeer::doSelect($c);

	return frsPackagesToSoap($FrsPackages);
}

function frsPackagesToSoap($FrsPackages) {
	$return = array();

	foreach ($FrsPackages as $item) {
		$return[]=array(
				'package_id'=> $item->getFrsPackageId(),
				'name'		=> $item->getPackageName(),
				'status_id' => $item->getStatusId(),
				'is_public' => $item->getIsPublic()
		);
	}
	return $return;
}

function addPackage($session_ser, $group_id, $package_name, $is_public) {
	$session = continueSession($session_ser);

	$Project = ProjectPeer::retrieveByPk($group_id);
	if (!$Project || !is_object($Project)) {
		return new soap_fault ('', 'addPackage', 'Could Not Get Group', 'Could Not Get Group');
	}
	
	if ($is_public <> 0) {
		$is_public = 1;
	} else {
		$is_public = 0;
	}

	$FrsPackage = new FrsPackage();
	$FrsPackage->setProject($Project);
	$FrsPackage->setPackageName(Validator::sanitizeString($package_name));
    $FrsPackage->setStatusId(0);
    $FrsPackage->setIsPublic($is_public);
    $FrsPackage->setRequireLogin(0);

    try {
		$FrsPackage->save();
		return $FrsPackage->getFrsPackageId();
	} catch (Exception $ex) {
		return new soap_fault ('', 'addPackage', $ex->getMessage(), $ex->getMessage());
	}
}

function getReleases($session_ser, $group_id, $package_id) {
	$session = continueSession($session_ser);

	$c = new Criteria();
	$c->add(FrsPackagePeer::PROJECT_ID, $group_id);
	$c->add(FrsPackagePeer::FRS_PACKAGE_ID, $package_id);
	$FrsPackage = FrsPackagePeer::doSelectOne($c);

	if (!is_object($FrsPackage) || !$FrsPackage->userCanRead()) {
		return new soap_fault ('', 'getReleases', 'Could Not Get Releases: Permission Denied',
							'Could Not Get Releases: Permission Denied');
	}

	$c = new Criteria();
	$c->add(FrsReleasePeer::FRS_PACKAGE_ID, $FrsPackage->getFrsPackageId());
	$FrsReleases = FrsReleasePeer::doSelect($c);
	return frsReleaseToSoap($FrsReleases);
}

function frsReleaseToSoap($FrsReleases) {
	$return = array();

	foreach ($FrsReleases as $item) {
		$return[]=array(
					'release_id' => $item->getFrsReleaseId(),
					'name' => $item->getReleaseName(),
					'notes' => $item->getReleaseNotes(),
					'changes' => $item->getChanges(),
					'status_id' => $item->getStatusId(),
					'release_date' => $item->getReleaseDate()
		);
	}
	return $return;
}

function addRelease($session_ser, $group_id, $package_id, $name, $notes, $changes, $release_date) {
	$session = continueSession($session_ser);

	$c = new Criteria();
	$c->add(FrsPackagePeer::PROJECT_ID, $group_id);
	$c->add(FrsPackagePeer::FRS_PACKAGE_ID, $package_id);
	$FrsPackage = FrsPackagePeer::doSelectOne($c);

	if (!is_object($FrsPackage) || !$FrsPackage->userCanRead()) {
		return new soap_fault ('', 'addRelease', 'Could Not Add Release: Permission Denied',
							'Could Not Add Release: Permission Denied');
	}
	
	$FrsRelease = new FrsRelease();
	$FrsRelease->setFrsPackage($FrsPackage);

	$FrsRelease->setReleaseName(Validator::sanitizeString($name));
	$FrsRelease->setReleaseNotes(Validator::sanitizeString($notes));
	$FrsRelease->setChanges(Validator::sanitizeString($changes));
	$FrsRelease->setStatusId(FrsRelease::STATUS_ACTIVE);
	$FrsRelease->setPreformatted(Validator::sanitizeInt(0));
	$FrsRelease->setReleaseDate(Validator::sanitizeDate($release_date));
	$FrsRelease->setReleasedBy($session->getUser()->getUserId());

    try {
		$FrsRelease->save();
		return $FrsRelease->getFrsReleaseId();
	} catch (Exception $ex) {
		return new soap_fault ('', 'addRelease', $ex->getMessage(), $ex->getMessage());
	}
}

function getFiles($session_ser, $group_id, $package_id, $release_id) {
	$session = continueSession($session_ser);

	$c = new Criteria();
	$c->add(FrsReleasePeer::FRS_RELEASE_ID, $release_id);
	$c->add(FrsReleasePeer::FRS_PACKAGE_ID, $package_id);
	// Do we really need this?
	$c->addJoin(FrsPackagePeer::PROJECT_ID, ProjectPeer::PROJECT_ID);
	$c->addJoin(FrsPackagePeer::FRS_PACKAGE_ID, FrsReleasePeer::FRS_PACKAGE_ID);
	$c->add(ProjectPeer::PROJECT_ID, $group_id);

	$FrsRelease = FrsReleasePeer::doSelectOne($c);

	if (!is_object($FrsRelease) || !$FrsRelease->userCanRead()) {
		return new soap_fault ('', 'getFiles', 'Could Not Get Files: Permission Denied',
							'Could Not Get Files: Permission Denied');
	}
	
	$c = new Criteria();
	$c->add(FilesystemPeer::SECTION, FrsRelease::SECTION);
	$c->add(FilesystemPeer::REF_ID, $FrsRelease->getFrsReleaseId());
	$Files = FilesystemPeer::doSelect($c);
		
	return filesToSoap($Files);
}

function filesToSoap($Files) {
	$return = array();

	foreach ($Files as $file) {
		$return[]=array(
			'file_id' => $file->getFilesystemId(),
			'name' => $file->getFileName(),
			'size' => $file->getFileSize(),
			'type' => $file->getFileType(),
			'processor' => '',
			'downloads' => $file->getDownloadCount(),
			'release' => 0,
			'date' => 0,
		);
	}
	return $return;
}

function getFile($session_ser, $group_id, $package_id, $release_id, $file_id) {

	$session = continueSession($session_ser);

	$c = new Criteria();
	$c->add(FrsReleasePeer::FRS_RELEASE_ID, $release_id);
	$c->add(FrsReleasePeer::FRS_PACKAGE_ID, $package_id);
	// Do we really need this?
	$c->addJoin(FrsPackagePeer::PROJECT_ID, ProjectPeer::PROJECT_ID);
	$c->addJoin(FrsPackagePeer::FRS_PACKAGE_ID, FrsReleasePeer::FRS_PACKAGE_ID);
	$c->add(ProjectPeer::PROJECT_ID, $group_id);

	$FrsRelease = FrsReleasePeer::doSelectOne($c);

	if (!is_object($FrsRelease) || !$FrsRelease->userCanRead()) {
		return new soap_fault ('', 'getFile', 'Could Not Get File: Permission Denied',
							'Could Not Get File: Permission Denied');
	}
	
	$c = new Criteria();
	$c->add(FilesystemPeer::FILESYSTEM_ID, $file_id);
	$c->add(FilesystemPeer::SECTION, FrsRelease::SECTION);
	$c->add(FilesystemPeer::REF_ID, $FrsRelease->getFrsReleaseId());
	$File = FilesystemPeer::doSelectOne($c);

	if (!$File || !is_object($File)) {
		return new soap_fault ('', 'getFile', 'Could Not Get File: File doesn\'t exit',
							'Could Not Get File: File doesn\'t exit');
	}
	return base64_encode($File->getFileData());
}

function addFile($session_ser, $group_id, $package_id, $release_id, $name,
			$base64_contents, $type_id, $processor_id, $release_time) {

	$session = continueSession($session_ser);

	$c = new Criteria();
	$c->add(FrsReleasePeer::FRS_RELEASE_ID, $release_id);
	$c->add(FrsReleasePeer::FRS_PACKAGE_ID, $package_id);
	// Do we really need this?
	$c->addJoin(FrsPackagePeer::PROJECT_ID, ProjectPeer::PROJECT_ID);
	$c->addJoin(FrsPackagePeer::FRS_PACKAGE_ID, FrsReleasePeer::FRS_PACKAGE_ID);
	$c->add(ProjectPeer::PROJECT_ID, $group_id);

	$FrsRelease = FrsReleasePeer::doSelectOne($c);

	if (!is_object($FrsRelease) || !$FrsRelease->userCanRead()) {
		return new soap_fault ('', 'addFile', 'Could Not Add File: Permission Denied',
							'Could Not Add File: Permission Denied');
	}

	$contents = base64_decode($base64_contents);
	$fs = new Filesystem();
	$fs->setSection(FrsRelease::SECTION);
	$fs->setRefId($FrsRelease->getFrsReleaseId());
	$fs->setFileName($name);
	$fs->setFileType($type_id);
	$fs->setFileSize(strlen($contents));
	$fs->setPostedBy($session->getUser()->getUserId());

    try {
		$fs->save();
		$fs->setFileDataFromString($contents);
		return $fs->getFilesystemId();
	} catch (Exception $ex) {
		return new soap_fault ('', 'addFile', $ex->getMessage(), $ex->getMessage());
	}
}
?>