<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

// This code generates a SOAP API for handling Filesystem

require_once('core/Filesystem.php');
require_once('core/FilesystemPeer.php');
require_once('Validator.php');

/* ******************************************************************** */
/*                                 DATA TYPES                           */
/* ******************************************************************** */


$server->wsdl->addComplexType(
	'Filesystem',
	'complexType',
	'struct',
	'sequence',
	'',
	array(
		'filesystem_id' => array('name'=>'filesystem_id', 'type' => 'xsd:int'),
		'section' => array('name'=>'section', 'type' => 'xsd:string'),
		'ref_id' => array('name'=>'ref_id', 'type' => 'xsd:int'),
		'file_name' => array('name'=>'file_name', 'type' => 'xsd:string'),
		'file_name_safe' => array('name'=>'file_name_safe', 'type' => 'xsd:string'),
		'file_type' => array('name'=>'file_type', 'type' => 'xsd:string'),
		'file_size' => array('name'=>'file_size', 'type' => 'xsd:int'),
		'deleted' => array('name'=>'deleted', 'type' => 'xsd:boolean'),
		'posted_by' => array('name'=>'posted_by', 'type' => 'xsd:int'),
		'download_count' => array('name'=>'download_count', 'type' => 'xsd:int'),
		'download_url' => array('name'=>'download_url', 'type' => 'xsd:string')
	)
);

$server->wsdl->addComplexType(
	'FilesystemData',
	'complexType',
	'struct',
	'sequence',
	'',
	array(
		'filesystem_id' => array('name'=>'filesystem_id', 'type' => 'xsd:int'),
		'file_data' => array('name'=>'file_data', 'type' => 'xsd:string')
	)
);


$server->wsdl->addComplexType(
	'ArrayOfFilesystem',
	'complexType',
	'array',
	'',
	'SOAP-ENC:Array',
	array(),
	array(array('ref' => 'SOAP-ENC:arrayType', 'wsdl:arrayType' => 'tns:Filesystem[]')),
	'tns:Filesystem'
);

$server->wsdl->addComplexType(
	'ArrayOfFilesystemData',
	'complexType',
	'array',
	'',
	'SOAP-ENC:Array',
	array(),
	array(array('ref' => 'SOAP-ENC:arrayType', 'wsdl:arrayType' => 'tns:FilesystemData[]')),
	'tns:FilesystemData'
);

/* ******************************************************************** */
/*                                 METHODS                              */
/* ******************************************************************** */

$server->register(
	'getFilesystems',
	array(
		'sessionHash'	=> 'xsd:string',
		'section' => 'xsd:string',
		'ref_id' => 'xsd:int'
	), 
	array('getFilesystems' => 'tns:ArrayOfFilesystem'),
	$uri, 
	$uri.'#getFilesystems',
	'rpc',
	'encoded'
);


/**
 * SOAP method to retrieve a list of Filesystem
 * @param	$filesystem_id	Filter by filesystem_id
 * @param	$section	Filter by section
 * @param	$ref_id	Filter by ref_id
 */
function getFilesystems($sessionHash, $section, $ref_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$c = new Criteria();
	
	if (!empty($section)) {
		$c->add(FilesystemPeer::SECTION, $section);
	}
	if ($ref_id >= 0) {
		$c->add(FilesystemPeer::REF_ID, $ref_id);
	}
	
	$objs = FilesystemPeer::doSelect($c);
	$res = array();
	foreach ($objs as $obj) {
		if (checkFSPermission($obj->getSection(), $obj->getRefId())) {
			$res[] = FilesystemToSOAP($obj);
		}
	}
	
	return $res;
	
}

$server->register(
	'getFilesystemArray',
	array(
		'sessionHash'	=> 'xsd:string',
		'filesystem_id' => 'tns:ArrayOfInt',
	), 
	array('getFilesystemArray' => 'tns:ArrayOfFilesystem'),
	$uri, 
	$uri.'#getFilesystemArray',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve an array of Filesystem
 * @param	$filesystem_id	Retrieve the Filesystem array identified by the array of filesystem_id
 */
function getFilesystemArray($sessionHash, $filesystem_idArr) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$ret = array();

	$crit=new Criteria();
        $crit->add(FilesystemPeer::FILESYSTEM_ID,$filesystem_idArr,Criteria::IN);
	$objs = FilesystemPeer::doSelect($crit);
	foreach ($objs as $obj) {
		if (checkFSPermission($obj->getSection(), $obj->getRefId())) {
			$ret[] = FilesystemToSOAP($obj);
		}
	}
	return $ret;
}


$server->register(
	'getFilesystemDataArray',
	array(
		'sessionHash'	=> 'xsd:string',
		'filesystem_id' => 'tns:ArrayOfInt',
	), 
	array('getFilesystemData' => 'tns:ArrayOfFilesystemData'),
	$uri, 
	$uri.'#getFilesystemData',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve an array of Filesystem Data
 * @param	$filesystem_id	Retrieve the Filesystem Data array identified by the array of filesystem_id
 */
function getFilesystemDataArray($sessionHash, $filesystem_idArr) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$ret = array();

	$crit=new Criteria();
        $crit->add(FilesystemPeer::FILESYSTEM_ID,$filesystem_idArr,Criteria::IN);
	$objs = FilesystemPeer::doSelect($crit);
	foreach ($objs as $obj) {
		if (checkFSPermission($obj->getSection(), $obj->getRefId())) {
			$ret[] = FilesystemDataToSOAP($obj);
		}
	}
	return $ret;
}


$server->register(
	'getFilesystem',
	array(
		'sessionHash'	=> 'xsd:string',
		'filesystem_id' => 'xsd:int',
	), 
	array('getFilesystem' => 'tns:Filesystem'),
	$uri, 
	$uri.'#getFilesystem',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve a single Filesystem
 * @param	$filesystem_id	Retrieve the Filesystem identified by this filesystem_id
 */
function getFilesystem($sessionHash, $filesystem_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$obj = FilesystemPeer::retrieveByPK($filesystem_id);
	if (is_null($obj)) {
		return null;
	}
	if (!checkFSPermission($obj->getSection(), $obj->getRefId())) {
		return new soap_fault('', 'getFilesystem','Permission Denied - Cannot View This Record','Permission Denied - Cannot View This Record');
	}
	return FilesystemToSOAP($obj);
}


$server->register(
	'getFilesystemData',
	array(
		'sessionHash'	=> 'xsd:string',
		'filesystem_id' => 'xsd:int',
	), 
	array('getFilesystemData' => 'tns:FilesystemData'),
	$uri, 
	$uri.'#getFilesystemData',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve a single Filesystem Data
 * @param	$filesystem_id	Retrieve the Filesystem Data identified by this filesystem_id
 */
function getFilesystemData($sessionHash, $filesystem_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$obj = FilesystemPeer::retrieveByPK($filesystem_id);
	if (is_null($obj)) {
		return null;
	}
	if (!checkFSPermission($obj->getSection(), $obj->getRefId())) {
		return new soap_fault('', 'getFilesystemData','Permission Denied - Cannot View This Record','Permission Denied - Cannot View This Record');
	}
	return FilesystemDataToSOAP($obj);
}


$server->register(
	'deleteFilesystem',
	array(
		'sessionHash'	=> 'xsd:string',
		'filesystem_id' => 'xsd:int',
	), 
	array('deleteFilesystem' => 'xsd:int'),
	$uri, 
	$uri.'#deleteFilesystem',
	'rpc',
	'encoded'
);

/**
 * SOAP method to delete a single Filesystem
 * @param	$filesystem_id	Delete the Filesystem identified by this filesystem_id
 */
function deleteFilesystem($sessionHash, $filesystem_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$obj = FilesystemPeer::retrieveByPK($filesystem_id);
	if (is_null($obj)) {
		return new soap_fault('', 'deleteFilesystem','Permission Denied - Cannot Delete This Record: Cannot View This Record','Permission Denied - Cannot View This Record');
	}
	if (!checkFSPermission($obj->getSection(), $obj->getRefId(), 'delete')) {
		return new soap_fault('', 'deleteFilesystem','Permission Denied - Cannot Delete This Record: Cannot Delete This Record','Permission Denied - Cannot Delete This Record');
	}
	try {
		$obj->delete();
	} catch (Exception $e) {
		return new soap_fault('', 'deleteFilesystem','Permission Denied - Cannot Delete This Record:' . $e->getMessage(),$e->getMessage());
	}
	return 1;
}


/*
$server->register(
	'updateFilesystem',
	array(
		'sessionHash'	=> 'xsd:string',
		'filesystem_id' => 'xsd:int',
		'section' => 'xsd:string',
		'ref_id' => 'xsd:int',
		'file_name' => 'xsd:string',
		'file_name_safe' => 'xsd:string',
		'file_type' => 'xsd:string',
		'file_size' => 'xsd:int',
		'deleted' => 'xsd:boolean',
		'posted_by' => 'xsd:int',
		'download_count' => 'xsd:int',
		'strings' => 'xsd:string',
	), 
	array('updateFilesystem' => 'xsd:int'),
	$uri, 
	$uri.'#updateFilesystem',
	'rpc',
	'encoded'
);

/**
 * SOAP method to update Filesystem
 * @param	$filesystem_id	 Filesystem to update
 * @param	$section
 * @param	$ref_id
 * @param	$file_name
 * @param	$file_name_safe
 * @param	$file_type
 * @param	$file_size
 * @param	$deleted
 * @param	$posted_by
 * @param	$download_count
 * @param	$strings
 * /
function updateFilesystem($sessionHash, $filesystem_id, $section, $ref_id, $file_name, $file_name_safe, $file_type, $file_size, $deleted, $posted_by, $download_count, $strings) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$obj = FilesystemPeer::retrieveByPK($filesystem_id);
	if (is_null($obj)) {
		return new soap_fault('', 'updateFilesystem','Couldn\'t update Filesystem: invalid filesystem_id '.$filesystem_id);
	}
	
	if (!is_null($section)) {
		$obj->setSection( Validator::sanitizeString($section) );
	}
	if (!is_null($ref_id)) {
		$obj->setRefId( Validator::sanitizeInt($ref_id) );
	}
	if (!is_null($file_name)) {
		$obj->setFileName( Validator::sanitizeString($file_name) );
	}
	if (!is_null($file_name_safe)) {
		$obj->setFileNameSafe( Validator::sanitizeString($file_name_safe) );
	}
	if (!is_null($file_type)) {
		$obj->setFileType( Validator::sanitizeString($file_type) );
	}
	if (!is_null($file_size)) {
		$obj->setFileSize( Validator::sanitizeInt($file_size) );
	}
	if (!is_null($deleted)) {
		$obj->setDeleted( Validator::sanitizeBool($deleted) );
	}
	if (!is_null($posted_by)) {
		$obj->setPostedBy( Validator::sanitizeInt($posted_by) );
	}
	if (!is_null($download_count)) {
		$obj->setDownloadCount( Validator::sanitizeInt($download_count) );
	}
	if (!is_null($strings)) {
		$obj->setStrings( Validator::sanitizeString($strings) );
	}
	
	try {
		$obj->save();
	} catch (Exception $e) {
		return new soap_fault('', 'updateFilesystem','Permission Denied - Cannot update This Record',$e->getMessage());
	}
	return 1;
}
*/


$server->register(
	'addFilesystem',
	array(
		'sessionHash'	=> 'xsd:string',
		'section' => 'xsd:string',
		'ref_id' => 'xsd:int',
		'file_name' => 'xsd:string',
		'file_type' => 'xsd:string',
		'base64_file_data' => 'xsd:string'
	), 
	array('addFilesystem' => 'xsd:int'),
	$uri, 
	$uri.'#getFilesystems',
	'rpc',
	'encoded'
);


/**
 * SOAP method to add a Filesystem
 * @param	$section
 * @param	$ref_id
 * @param	$file_name
 * @param	$file_type
 * @param	$base64_file_data
 */
function addFilesystem($sessionHash, $section, $ref_id, $file_name, $file_type, $base64_file_data) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	if (!checkFSPermission($section, $ref_id, 'create')) {
		return new soap_fault('', 'addFilesystem','Permission Denied - Cannot add This Record', 'Permission Denied - Cannot add This Record');
	}

	$contents = base64_decode($base64_file_data);
	
	$obj = new Filesystem();
	
	$file_name = Validator::sanitizeString($file_name);
	$file_type = Validator::sanitizeString($file_type);
	$obj->setSection( Validator::sanitizeString($section) );
	$obj->setRefId( Validator::sanitizeInt($ref_id) );
	$obj->setFileName( $file_name );
	$obj->setFileSize( strlen($contents) );
	$obj->setPostedBy( Session::getSession()->getUser()->getUserId() );

	try {
		$obj->save();
		$file_type = ($file_type == '') ? @mime_content_type($obj->getFilePath()) : $file_type;
		$obj->setFileType( $file_type );
		$obj->setFileDataFromString( $contents );
		return $obj->getFilesystemId();
	} catch (Exception $e) {
		return new soap_fault('', 'addFilesystem','Permission Denied - Cannot add This Record: ' . $e->getMessage(),$e->getMessage());
	}
}

function checkFSPermission($section, $ref_id, $type='') {

	switch($section) {
		case FrsRelease::SECTION :
			$obj = FrsReleasePeer::retrieveByPK($ref_id);
			break;
		case DocmanFileVersion::SECTION :
			$obj = DocmanFileVersionPeer::retrieveByPK($ref_id);
			break;
		case TrackerItem::SECTION:
			$obj = TrackerItemPeer::retrieveByPk($ref_id);
			break;
		case Wiki::WIKI_FILES_SECTION:
			$obj = WikiPagePeer::retrieveByPk($ref_id);
			break;
		case ForumMessage::SECTION:
			$obj = ForumMessagePeer::retrieveByPk($ref_id);
			break;
		default:
			return false;
			break;
	}

	if (isset($obj) && $obj != null) {
		if ($type=='delete') {
			return $obj->userCanDelete();
		} else if ($type=='create') {
			return $obj->userCanCreate();
		} else if ($type=='update') {
			return $obj->userCanUpdate();
		} else {
			return $obj->userCanRead();
		}
	} else {
		$myerror = 'invalid object';
		return false;
	}
}

function FilesystemToSOAP(&$obj) {
	$ret = array();
	$ret['filesystem_id'] = $obj->getFilesystemId();
	$ret['section'] = $obj->getSection();
	$ret['ref_id'] = $obj->getRefId();
	$ret['file_name'] = $obj->getFileName();
	$ret['file_name_safe'] = $obj->getFileNameSafe();
	$ret['file_type'] = $obj->getFileType();
	$ret['file_size'] = $obj->getFileSize();
	$ret['deleted'] = $obj->getDeleted();
	$ret['posted_by'] = $obj->getPostedBy();
	$ret['download_count'] = $obj->getDownloadCount();
	$ret['download_url'] = GFUrl::getURL()->downloadURL($obj->getSection(), $obj->getRefId(), $obj->getFilesystemId(), $obj->getFileNameSafe());
	return $ret;
}

function FilesystemDataToSOAP(&$obj) {
	$ret = array();
	$ret['filesystem_id'] = $obj->getFilesystemId();
	$ret['file_data'] = base64_encode($obj->getFileData());
	return $ret;
}
?>
