<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

// This code generates a SOAP API for handling TrackerQueryField

require_once('plugins/tracker/lib/TrackerQueryField.php');
require_once('plugins/tracker/lib/TrackerQueryFieldPeer.php');
require_once('Validator.php');

/* ******************************************************************** */
/*                                 DATA TYPES                           */
/* ******************************************************************** */
$server->wsdl->addComplexType(
	'TrackerQueryField',
	'complexType',
	'struct',
	'sequence',
	'',
	array(
		'tracker_query_id' => array('name'=>'tracker_query_id', 'type' => 'xsd:int'),
		'query_field_type' => array('name'=>'query_field_type', 'type' => 'xsd:string'),
		'query_field_id' => array('name'=>'query_field_id', 'type' => 'xsd:int'),
		'query_field_values' => array('name'=>'query_field_values', 'type' => 'xsd:string'),
	)
);


$server->wsdl->addComplexType(
	'ArrayOfTrackerQueryField',
	'complexType',
	'array',
	'',
	'SOAP-ENC:Array',
	array(),
	array(array('ref' => 'SOAP-ENC:arrayType', 'wsdl:arrayType' => 'tns:TrackerQueryField[]')),
	'tns:TrackerQueryField'
);

/* ******************************************************************** */
/*                                 METHODS                              */
/* ******************************************************************** */

$server->register(
	'getTrackerQueryFields',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_query_id' => 'xsd:int'
	), 
	array('getTrackerQueryFields' => 'tns:ArrayOfTrackerQueryField'),
	$uri, 
	$uri.'#getTrackerQueryFields',
	'rpc',
	'encoded'
);


/**
 * SOAP method to retrieve a list of TrackerQueryField
 * @param	$tracker_query_id	Filter by tracker_query_id
 * @param	$query_field_type	Filter by query_field_type
 * @param	$query_field_id	Filter by query_field_id
 */
function getTrackerQueryFields($sessionHash, $tracker_query_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	if (!checkQueryPermission($tracker_query_id)) {
		 return new soap_fault('', 'getTrackerQueryFields','Permission Denied - Cannot View These Records','Permission Denied - Cannot View This Record');
	}
	$c = new Criteria();
	$c->add(TrackerQueryFieldPeer::TRACKER_QUERY_ID, $tracker_query_id);
	$objs = TrackerQueryFieldPeer::doSelect($c);
	$res = array();
	foreach ($objs as $obj) {
		$res[] = TrackerQueryFieldToSOAP($obj);
	}
	return $res;
}

$server->register(
	'deleteTrackerQueryField',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_query_id' => 'xsd:int',
		'query_field_type' => 'xsd:string',
		'query_field_id' => 'xsd:int',
	), 
	array('deleteTrackerQueryField' => 'xsd:int'),
	$uri, 
	$uri.'#deleteTrackerQueryField',
	'rpc',
	'encoded'
);


/**
 * SOAP method to add a TrackerQueryField
 * @param	$tracker_query_id
 * @param	$query_field_type
 * @param	$query_field_id
 */
function deleteTrackerQueryField($sessionHash, $tracker_query_id, $query_field_type, $query_field_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	if (!checkQueryPermission($tracker_query_id)) {
		 return new soap_fault('', 'deleteTrackerQueryField','Permission Denied - Cannot Delete This Record','Permission Denied - Cannot Delete This Record');
	}

	$c = new Criteria();
	$c->add(TrackerQueryFieldPeer::TRACKER_QUERY_ID, $tracker_query_id);

	if ($query_field_type != '') {
		$c->add(TrackerQueryFieldPeer::QUERY_FIELD_TYPE, $query_field_type);
	}
	if ($query_field_id != '') {
		$c->add(TrackerQueryFieldPeer::QUERY_FIELD_ID, $query_field_id);
	}
	$obj = TrackerQueryFieldPeer::doSelectOne($c);
	if (is_null($obj)) {
		return new soap_fault('', 'deleteTrackerQueryField','Permission Denied - Cannot Delete This Record','Permission Denied - Cannot Delete This Record');
	}
	try {
		$obj->delete();
	} catch (Exception $e) {
		return new soap_fault('', 'deleteTrackerQueryField','Permission Denied - Cannot Delete This Record',$e->getMessage());
	}
	return 1;
}


$server->register(
	'addTrackerQueryField',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_query_id' => 'xsd:int',
		'query_field_type' => 'xsd:string',
		'query_field_id' => 'xsd:int',
		'query_field_values' => 'xsd:string',
	), 
	array('addTrackerQueryField' => 'xsd:int'),
	$uri, 
	$uri.'#addTrackerQueryField',
	'rpc',
	'encoded'
);


/**
 * SOAP method to add a TrackerQueryField
 * @param	$tracker_query_id
 * @param	$query_field_type
 * @param	$query_field_id
 * @param	$query_field_values
 */
function addTrackerQueryField($sessionHash, $tracker_query_id, $query_field_type, $query_field_id, $query_field_values) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	if (!checkQueryPermission($tracker_query_id)) {
		 return new soap_fault('', 'addTrackerQueryField','Permission Denied - Cannot add This Record','Permission Denied - Cannot add This Record');
	}

	$obj = new TrackerQueryField();
	$obj->setTrackerQueryId( Validator::sanitizeInt($tracker_query_id) );
	$obj->setQueryFieldId( Validator::sanitizeInt($query_field_id) );
	$obj->setQueryFieldType( Validator::sanitizeString($query_field_type) );
	$obj->setQueryFieldValues( Validator::sanitizeString($query_field_values) );
	try {
		$obj->save();
		return 1;
	} catch (Exception $e) {
		return new soap_fault('', 'addTrackerQueryField','Permission Denied - Cannot add This Record',$e->getMessage());
	}
}

function TrackerQueryFieldToSOAP(&$obj) {
	$ret = array();
	$ret['tracker_query_id'] = $obj->getTrackerQueryId();
	$ret['query_field_type'] = $obj->getQueryFieldType();
	$ret['query_field_id'] = $obj->getQueryFieldId();
	$ret['query_field_values'] = $obj->getQueryFieldValues();
	return $ret;
}
?>