<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2006-02-27
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

//
// This code creates a new User and saves it to the 
// database
//
require_once('Validator.php');
require_once('SystemUtils.php');
require_once('exceptions/InvalidDataException.php');
require_once('lib/core/User.php');
require_once('session/Session.php');
require_once('securimage/securimage.php');

$User = new User();
$session = Session::getSession();

if ($session->isLoggedIn()) {
	// if user is already logged in, go to the logout page
	header('Location: /gf/account/?action=Logout');
}

$errors = array();

if ($_SERVER['REQUEST_METHOD'] != 'POST') {
	throw new PermissionDeniedException();
}

if (!isset($_POST['password']) || !isset($_POST['password_confirm'])) {
	$errors['password'] = Language::getSessText('User.EmptyPassword');
}
// note that we make this validation _outside_ the User class on purpose
if ($_POST['password'] != $_POST['password_confirm']) {
	$errors['password'] = Language::getSessText('User.PasswordsDontMatch');
}
if (GFConf::get('system.userRegClickLicense')) {
	if (isset($_REQUEST['iagree']) && Validator::sanitizeBool($_REQUEST['iagree'])) {
	} else {
		$errors['license']=Language::getSessText('License.DidntCheck');
	}
}

//if captcha is enabled, checks captcha code entered
if (GFConf::get('system.useCaptcha') == true) { 
	//Get form captcha
	$captcha_code = isset($_POST['captcha_code']) ? $_POST['captcha_code'] : 0;
	$securimage = new Securimage();
	//Verify captcha
	if ($securimage->check($captcha_code) == false) {
		$errors['captcha']=Language::getSessText('Captcha.DontMatch');
	}
}

if (count($errors) == 0) {
	$User->setUnixName( isset($_POST['unix_name']) ? Validator::sanitizeString($_POST['unix_name']) : '' );
	$User->setEmail( isset($_POST['email']) ? Validator::sanitizeString($_POST['email']) : '' );
	$User->setPassword( $_POST['password'] );		// Password must not be sanitized
	$User->setTimezone( isset($_POST['timezone']) ? Validator::sanitizeString($_POST['timezone']) : '' );
	$User->setDateFormat( isset($_POST['date_format']) ? Validator::sanitizeString($_POST['date_format']) : '' );
	$User->setAddress( isset($_POST['address']) ? Validator::sanitizeString($_POST['address']) : '' );
	$User->setAddress2( isset($_POST['address2']) ? Validator::sanitizeString($_POST['address2']) : '' );
	$User->setPhone( isset($_POST['phone']) ? Validator::sanitizeString($_POST['phone']) : '' );
	$User->setFax( isset($_POST['fax']) ? Validator::sanitizeString($_POST['fax']) : '' );
	$User->setTitle( isset($_POST['title']) ? Validator::sanitizeString($_POST['title']) : '' );
	$User->setFirstname( isset($_POST['firstname']) ? Validator::sanitizeString($_POST['firstname']) : '' );
	$User->setLastname( isset($_POST['lastname']) ? Validator::sanitizeString($_POST['lastname']) : '' );
	$User->setCcode( isset($_POST['ccode']) ? Validator::sanitizeString($_POST['ccode']) : '' );
	$User->setLanguageId( isset($_POST['language_id']) ? Validator::sanitizeInt($_POST['language_id']) : '' );
	$User->setThemeId( isset($_POST['theme_id']) ? Validator::sanitizeInt($_POST['theme_id']) : '' );
	$User->setConfirmHash(substr(md5(Session::getSession()->getSessionHash() . $_POST['password'] . time()),0,16));
	
	try {
		$User->save();
		
		// If the administrator doesn't have to approve this account, send the account confirmation
		// email
		if (GFConf::get('system.approveUsers') == false) {
			$User->sendRegistrationEmail();
		}

		GFUrl::getUrl()->redirect(GFUrl::getUrl()->currentBaseURL('PostUserRegistration'));
	} catch (InvalidDataException $ex) {
		$errors = array_merge($errors, $ex->getErrors());		// attach object errors to validation errors
		$feedback = '* '.implode('<br />* ', $errors);
		include('UserAdd.php');
	}
} else {
	$feedback = '* '.implode('<br />* ', $errors);
	include('UserAdd.php');	
}

?>
