<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

//
// This code creates a new Project and saves it to the 
// database
//
require_once('Validator.php');
require_once('exceptions/InvalidDataException.php');
require_once('core/Project.php');
require_once('exceptions/NotLoggedInException.php');
require_once('lib/GFExportImport.php');

$session = Session::getSession();
$Lang = $session->getLanguage();
if (!$session->isLoggedIn()) {
	throw new NotLoggedInException();
}

if ($_SERVER['REQUEST_METHOD'] != 'POST' || !$session->isSiteAdmin()) {
	throw new PermissionDeniedException();
}

if (!isset($_REQUEST['exportSelector']) || !isset($_REQUEST['importProjectName'])) {
	include('ProjectImport.php');
	exit();
}

$_REQUEST['exportSelector'] = Validator::sanitizeString($_REQUEST['exportSelector']);
$_REQUEST['importProjectName'] = Validator::sanitizeString($_REQUEST['importProjectName']);


// Verify that the imported project name doesnt exist

$project = ProjectPeer::retrieveByUnixName($_REQUEST['importProjectName']);
if(is_object($project)) {
	$feedback = $Lang->getText('ProjectTemplate.ProjectExists');
	include('ProjectImport.php');
	exit();
}


// exportSelector has the format "projectName - dateString"
$values = explode(' - ', $_REQUEST['exportSelector']);

if (!isset($_REQUEST['confirm']) || !Validator::sanitizeBool($_REQUEST['confirm'])) {
	$feedback = $Lang->getText('ProjectTemplate.DidntConfirm');
	include('ProjectImport.php');
	exit();
}
if (count($values)!= 2 || $values[0] == '' || $values[1] =='') {
	include('ProjectImport.php');
	exit();
}

$projectName = $values[0];
$dateString = $values[1];

if (isset($_REQUEST['importMode'])) {
	$importMode = Validator::sanitizeString($_REQUEST['importMode']);
}
else {
	$importMode='standard';
}

$exportDir = SystemUtils::buildFilePath(GFConf::get('system.exportProjects'), $projectName);
$exportDir = SystemUtils::buildFilePath($exportDir, $dateString);

$ei = new GFExportImport();
$ei->import($exportDir, false, false, "PROJECT_FULLIMPORT", $_REQUEST['importProjectName']);

/*
switch($importMode) {
	case 'overwrite':
		$ei->import($exportDir, true, false, "PROJECT_FULLIMPORT");
		break;
	case 'merge';
		$ei->import($exportDir, false, true, "PROJECT_FULLIMPORT");
		break;
	default:
		break;
}

*/
$feedback = '* '.$Lang->getText('ProjectTemplate.SuccessfulImport');
include('ProjectImport.php');

?>
